# -*- coding: utf-8 -*-
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements


class IFirefoxPortlet(IPortletDataProvider):
    """
    Firefox portlet interface
    """


class Assignment(base.Assignment):
    implements(IFirefoxPortlet)


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('firefox.pt')

    @property
    def available(self):
        return 'Firefox' not in self.request.get('HTTP_USER_AGENT', None)


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()



# -*- coding: utf-8 -*-
from cpskin.core.testing import CPSKIN_CORE_ROBOT_TESTING
# from plone.app.robotframework.utils import disableCSRFProtection
from plone.testing import layered

import robotsuite
import unittest


def test_suite():
    # disableCSRFProtection()
    suite = unittest.TestSuite()
    suite.addTests([
        layered(robotsuite.RobotTestSuite('robot'),
                layer=CPSKIN_CORE_ROBOT_TESTING),
    ])
    return suite



# -*- coding: utf-8 -*-
from datetime import datetime
from plone import api
from plone.formwidget.datetime.z3cform.widget import DatetimeWidget
from plone.formwidget.datetime.z3cform.widget import DateWidget
from Products.CMFCore.utils import getToolByName
from Products.PluginIndexes.KeywordIndex.KeywordIndex import KeywordIndex


def related2brains(self, related):
    """Return a list of brains based on a list of relations. Will filter
    relations if the user has no permission to access the content.

    :param related: related items
    :type related: list of relations
    :return: list of catalog brains
    """
    catalog = getToolByName(self.context, 'portal_catalog')
    brains = []
    for r in related:
        path = r.to_path
        if not path:
            # See #18546 for broken related items
            continue
        # the query will return an empty list if the user has no
        # permission to see the target object
        brains.extend(catalog(path=dict(query=path, depth=0)))
    return brains


def afterMemberAdd(self, member, id, password, properties):
    """
    Called by portal_registration.addMember() after a member
    has been added successfully
    """
    # if this is an auto registration, not an administrator action
    groups_tool = api.portal.get_tool('portal_groups')
    group_id = 'citizens'
    if group_id not in groups_tool.getGroupIds():
        groups_tool.addGroup(group_id)
    if api.user.is_anonymous():
        api.group.add_user(groupname=group_id, user=member)


def keyword_apply_index(self, request, resultset=None):
    indexes = ('standardTags', 'iamTags', 'isearchTags', 'hiddenTags')
    for key in [k for k in indexes if k in request and self.id == k]:
        request_key = request.get(key, {})
        if hasattr(request_key, 'query'):
            query_value = request_key.get('query')
            if isinstance(query_value, unicode):
                request[key]['query'] = query_value.encode('utf8')
        elif getattr(request_key, 'get', None):
            query_value = request_key.get('query')
            if isinstance(query_value, (list, tuple)):
                values = []
                for value in query_value:
                    if isinstance(value, unicode):
                        values.append(value.encode('utf8'))
                    else:
                        values.append(value)
                request[key]['query'] = values
        else:
            if isinstance(request_key, unicode):
                request[key] = request_key.encode('utf8')
        if isinstance(request.get(key), dict):
            if request.get(key).get('query') is None:
                request[key]['query'] = []
            elif isinstance(request[key]['query'], unicode):
                request[key]['query'] = request[key]['query'].encode('utf8')
    return super(KeywordIndex, self)._apply_index(request, resultset)


def date_widget_years(self):
    try:
        current = int(self.year)
    except:
        current = -1

    value = datetime.now().year
    now = int(value)
    before = now + self.years_range[0]
    after  = now + self.years_range[1]
    year_range = range(*(before, after))
    return [{'value': x,
             'name': x,
             'selected': x == current}
            for x in year_range]


def date_widget_update(self):
    now = datetime.now()
    min_value = -10
    max_value = 10
    if self.field.min:
        min_value = self.field.min.year - now.year
    if self.field.max:
        max_value = self.field.max.year - now.year + 1
    self.years_range = (min_value, max_value)
    super(DateWidget, self).update()


def datetime_widget_update(self):
    now = datetime.now()
    min_value = -10
    max_value = 10
    if self.field.min:
        min_value = self.field.min.year - now.year
    if self.field.max:
        max_value = self.field.max.year - now.year + 1
    self.years_range = (min_value, max_value)
    super(DatetimeWidget, self).update()



# -*- coding: utf-8 -*-
from cpskin.locales import CPSkinMessageFactory as _
from Products.CMFDefault.formlib.schema import EmailLine
from zope import schema
from zope.interface import Interface


class ISendToManagerForm(Interface):
    """ Interface for describing the 'sendtomanager' form """

    send_from_address = EmailLine(
        title=_(u"label_send_from", default=u"From"),
        description=_(u"help_send_from", default=u"Your email address."),
        required=True,
    )

    comment = schema.Text(
        title=_(u"label_comment", default=u"Change request"),
        description=_(
            u"help_comment", default=u"Report an error or missing information."
        ),
        required=False,
    )

    captcha = schema.TextLine(title=u"ReCaptcha", description=u"", required=False)


class IBannerActivationView(Interface):
    """ Banner activation """

    can_enable_banner = schema.Bool(u"Can enable banner", readonly=True)
    can_disable_banner = schema.Bool(u"Can disable banner", readonly=True)

    def enable_banner():
        """ Enable banner
        """

    def disable_banner():
        """ Disable banner
        """

    can_enable_local_banner = schema.Bool(u"Can enable local banner", readonly=True)
    can_disable_local_banner = schema.Bool(u"Can disable local banner", readonly=True)

    def enable_local_banner():
        """ Enable local banner
        """

    def disable_local_banner():
        """ Disable local banner
        """


class IMediaActivationView(Interface):
    """ media activation """

    can_enable_media = schema.Bool(u"Can enable multimedia viewlet", readonly=True)
    can_disable_media = schema.Bool(u"Can disable multimedia viewlet", readonly=True)

    def enable_media():
        """ Enable multimedia viewlet
        """

    def disable_media():
        """ Disable multimedia viewlet
        """


class INavigationToggleView(Interface):
    """ navigation toggle activation """

    can_enable_navigation_toggle = schema.Bool(
        u"Can enable navigation toggle on folder", readonly=True
    )
    can_disable_navigation_toggle = schema.Bool(
        u"Can disable navigation toggle on folder", readonly=True
    )

    def enable_navigation_toggle():
        """ Enable navigation toggle on folder
        """

    def disable_navigation_toggle():
        """ Disable navigation toggle on folder
        """



# -*- coding: utf-8 -*-
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform.interfaces import IFormFieldProvider
from plone.namedfile.field import NamedImage
from plone.supermodel import model
from plone.supermodel.directives import fieldset
from zope.interface import provider


@provider(IFormFieldProvider)
class IOrganizationImages(model.Schema):

    fieldset(
        'images',
        label=_(u'Images'),
        fields=['image1', 'image2'],
    )

    image1 = NamedImage(
        title=_('Image 1'),
        description=_(u'This image is shown on directory view.'),
        required=False,
    )

    image2 = NamedImage(
        title=_('Image 2'),
        description=_(u'This image is shown on directory view when mouse hovers the card.'),  # noqa
        required=False,
    )



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from cpskin.core.browser.common import CommonView
from cpskin.core.interfaces import IFolderViewSelectedContent
from cpskin.core.interfaces import IFolderViewWithBigImages
from cpskin.core.utils import image_scale
from cpskin.core.vocabulary import DISPLAY_TYPES
from cpskin.locales import CPSkinMessageFactory as _
from datetime import datetime
from DateTime import DateTime
from imio.media.browser import utils
from plone import api
from plone.app.contenttypes.browser.folder import FolderView as FoldV
from plone.app.event.base import expand_events
from plone.app.event.base import filter_and_resort
from plone.app.event.base import RET_MODE_ACCESSORS
from plone.app.event.interfaces import IEventSettings
from plone.app.event.recurrence import RecurrenceSupport
from plone.app.querystring import queryparser
from plone.event.interfaces import IEvent
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import noLongerProvides

import json
import logging
import pytz

logger = logging.getLogger("cpskin.core")

ADDABLE_TYPES = ["Collection", "Document", "Folder", "rss_feed"]


class FolderView(FoldV, CommonView):
    def _redirect(self, msg=""):
        if self.request:
            if msg:
                api.portal.show_message(message=msg, request=self.request, type="info")
            self.request.response.redirect(self.context.absolute_url())
        return msg

    def _get_real_context(self):
        context = self.context
        plone_view = getMultiAdapter((context, self.request), name="plone")
        if plone_view.isDefaultPageInFolder():
            context = aq_parent(context)
        context = aq_inner(context)
        return context

    def isFolderViewActivated(self, context=None):
        """Check if folderview is activated on context"""
        if context is None:
            context = self.context
        layout = context.getLayout()
        if layout == "folderview":
            return True
        return False

    def can_configure(self):
        """Check if folderview can be configured on context"""
        context = self.context
        if not IFolderish.providedBy(context):
            return False
        already_activated = self.isFolderViewActivated()
        return not already_activated

    def configure(self):
        """Configure folders and collections for folderview"""
        context = self.context
        configure_folderviews(context)
        api.portal.show_message(
            message=_(u"Vue index avec collections configurée."),
            request=self.request,
            type="info",
        )
        self.request.response.redirect(context.absolute_url())
        return ""

    def is_event_collection(self, brains):
        if len(brains) > 0:
            obj = brains[0].getObject()
            return IEvent.providedBy(obj)
        return False

    def getResults(self, content, with_sticky=True):
        """Content is a Collection"""
        # Make a copy of the query to avoid modifying it
        query = list(content.query)
        index_view_keywords = getattr(content, "index_view_keywords", False)
        # set query for homepage
        if index_view_keywords:
            homepage_keywords = content.index_view_keywords
            query.append(
                {
                    "i": "hiddenTags",
                    "o": "plone.app.querystring.operation.selection.is",
                    "v": homepage_keywords,
                }
            )
        sort_on = getattr(content, "sort_on", None)
        sort_order = (
            "reverse" if getattr(content, "sort_reversed", False) else "ascending"
        )  # noqa
        sort_reversed = getattr(content, "sort_reversed", False)
        parsedquery = queryparser.parseFormquery(content, query, sort_on, sort_order)
        portal_catalog = api.portal.get_tool("portal_catalog")
        brains = portal_catalog(parsedquery)
        item_count_homepage = getattr(content, "item_count_homepage", 8)
        if self.is_event_collection(brains):
            start = DateTime()
            sort_on = getattr(content, "sort_on", "start")
            if sort_on in ("start", "end"):
                filter_and_resort_brains = filter_and_resort(
                    content, brains, start, None, sort_on, sort_reversed
                )
                brains = filter_and_resort_brains[:item_count_homepage]

        brains = brains[:item_count_homepage]
        if not with_sticky:
            return brains
        portal_catalog = api.portal.get_tool(name="portal_catalog")
        results = {"sticky-results": [], "standard-results": []}
        for brain in brains:
            if portal_catalog.getIndexDataForRID(brain.getRID())["is_sticky"]:
                results["sticky-results"].append(brain)
            else:
                results["standard-results"].append(brain)
        if not results["sticky-results"] and not results["standard-results"]:
            return None
        return results

    def getContents(self):
        brains = self.searchSelectedContent()
        objects = [brain.getObject() for brain in brains]
        realObjects = []
        for obj in objects:
            if obj.portal_type == "Folder":
                if obj.getDefaultPage() is not None:
                    realObject = getattr(obj, obj.getDefaultPage())
                    # check if realObject is a collection :
                    if getattr(realObject, "results", None):
                        if len(realObject.results()) > 0:
                            realObjects.append(realObject)
                else:
                    continue
            elif obj.portal_type == "rss_feed":
                realObjects.append(obj)
            elif obj.portal_type == "Link":
                try:
                    realObjects.append(
                        type(
                            "Link",
                            (object,),
                            {
                                "id": obj.id,
                                "render": self.context.unrestrictedTraverse(
                                    str(obj.remoteUrl)
                                ),
                                "portal_type": obj.portal_type,
                            },
                        )()
                    )
                except:
                    pass
            else:
                realObjects.append(obj)
        return realObjects

    def isBigImage(self, number, results, resultType):
        """
        Check if image should be big depending on position and result type
        (sticky / non-sticky)
        """
        if not self.bigImagesAreUsed():
            return False
        if resultType == "sticky-results" and number < 5:
            return True
        elif resultType == "standard-results" and number < 5 - len(
            results["sticky-results"]
        ):
            return True
        return False

    def getThumbSize(self, obj, isBigImage=False):
        prefix = "image"
        thumbSize = "thumb"
        if getattr(obj, "hasContentLeadImage", None):
            prefix = "leadImage"
        if isBigImage:
            thumbSize = "mini"
        return "%s_%s" % (prefix, thumbSize)

    def searchSelectedContent(self):
        path = "/".join(self.context.getPhysicalPath())
        portal_catalog = api.portal.get_tool("portal_catalog")
        queryDict = {}
        queryDict["path"] = {"query": path, "depth": 1}
        queryDict["portal_type"] = ADDABLE_TYPES
        queryDict["object_provides"] = IFolderViewSelectedContent.__identifier__  # noqa
        queryDict["sort_on"] = "getObjPositionInParent"
        queryDict["review_state"] = (
            "published_and_hidden",
            "published_and_shown",
            "published",
        )
        results = portal_catalog.searchResults(queryDict)
        return results

    def getSliderType(self, collection=""):
        if collection:
            return getattr(collection, "display_type", None)
        return None

    def hasFlexSlider(self):
        """Check if flexslider is available and installed"""
        try:
            from cpskin.slider.interfaces import ICPSkinSliderLayer
        except ImportError:
            return False
        else:
            request = getattr(self.context, "REQUEST", None)
            if ICPSkinSliderLayer.providedBy(request):
                return True
            return False

    def has_slick_slider(self):
        if self.hasFlexSlider() is True:
            return api.portal.get_registry_record(
                "cpskin.core.interfaces.ICPSkinSettings.use_slick",
                default=False,
            )
        return False

    def show_image(self, collection):
        slider_type = self.getSliderType(collection)
        return DISPLAY_TYPES[slider_type]["show-image"]

    def show_carousel(self, collection):
        slider_type = self.getSliderType(collection)
        return DISPLAY_TYPES[slider_type]["show-carousel"]

    def addContent(self):
        """Mark content to add it to folder view"""
        context = self._get_real_context()
        alsoProvides(context, IFolderViewSelectedContent)
        catalog = api.portal.get_tool("portal_catalog")
        catalog.reindexObject(context)
        self._redirect(_(u"Contenu ajouté à la vue index."))

    def removeContent(self):
        """Unmark content to remove it from folder view"""
        context = self._get_real_context()
        noLongerProvides(context, IFolderViewSelectedContent)
        catalog = api.portal.get_tool("portal_catalog")
        catalog.reindexObject(context)
        self._redirect(_(u"Contenu retiré de la vue index."))

    def isEligibleContent(self):
        context = self._get_real_context()
        if context.portal_type not in ADDABLE_TYPES:
            return False
        parent = aq_parent(context)
        if not self.isFolderViewActivated(parent):
            return False
        return True

    def canAddContent(self):
        if not self.isEligibleContent():
            return False
        context = self._get_real_context()
        if IFolderViewSelectedContent.providedBy(context):
            return False
        return True

    def canRemoveContent(self):
        if not self.isEligibleContent():
            return False
        context = self._get_real_context()
        if not IFolderViewSelectedContent.providedBy(context):
            return False
        return True

    def canUseBigImages(self):
        """Check if big images can be used on folder view"""
        if not self.isFolderViewActivated():
            return False
        context = self._get_real_context()
        return not IFolderViewWithBigImages.providedBy(context)

    def bigImagesAreUsed(self):
        context = self._get_real_context()
        return IFolderViewWithBigImages.providedBy(context)

    def canStopBigImagesUse(self):
        """Check if big images are used on folder view"""
        if not self.isFolderViewActivated():
            return False
        return self.bigImagesAreUsed()

    def useBigImages(self):
        """Use big images for first elements on folder view"""
        context = self._get_real_context()
        alsoProvides(context, IFolderViewWithBigImages)
        catalog = api.portal.get_tool("portal_catalog")
        catalog.reindexObject(context)
        self._redirect(_(u"Big images are now used on this folder view."))

    def stopBigImagesUse(self):
        """Use using big images for first elements on folder view"""
        context = self._get_real_context()
        noLongerProvides(context, IFolderViewWithBigImages)
        catalog = api.portal.get_tool("portal_catalog")
        catalog.reindexObject(context)
        self._redirect(_(u"Big images are not used anymore on this folder view."))

    def slick_config(self, content):
        portal_registry = getToolByName(self.context, "portal_registry")
        slider_timer = portal_registry[
            "cpskin.core.interfaces.ICPSkinSettings.slider_timer"
        ]
        min_items, max_items = self.get_items_number(content)
        slider_type = self.getSliderType(content)
        slider_config = DISPLAY_TYPES[slider_type]
        config = {
            "slidesToScroll": getattr(content, "slide_to_scroll", False),
            "dots": getattr(content, "show_dots", True),
            "arrows": getattr(content, "show_arrows", False),
            "speed": getattr(content, "speed", 300),
            "easing": slider_config.get("easing", "ease"),
            "autoplay": getattr(content, "autoplay_mode", False),
            "autoplaySpeed": getattr(content, "autoplay_speed", False),
            "centerMode": getattr(content, "use_center_mode", False),
            "centerPadding":str(getattr(content, "center_padding", False)) + "px",
            "fade": getattr(content, "fade", False),
        }
        if slider_config.get("variable-width", False):
            config["variableWidth"] = True
        else:
            config["responsive"] = [
                {
                    "breakpoint": getattr(content, "breakpoint_full", False),
                    "settings": {
                        "slidesToShow": getattr(content, "slides_to_show_full", False),
                        "slidesToScroll":getattr(content, "slides_to_scroll_full", False)
                    }
                },
                {
                    "breakpoint": getattr(content, "breakpoint_medium", False),
                    "settings": {
                        "slidesToShow": getattr(content, "slides_to_show_medium", False),
                        "slidesToScroll":getattr(content, "slides_to_scroll_medium", False)
                    }
                },
                {
                    "breakpoint": getattr(content, "breakpoint_small", False),
                    "settings": {
                        "slidesToShow": getattr(content, "slides_to_show_small", False),
                        "slidesToScroll":getattr(content, "slides_to_scroll_small", False)
                    }
                }
            ]
            config["slidesToShow"] = min_items
        return json.dumps(config)

    def slider_config(self, content):
        content_id = content.id
        portal_registry = getToolByName(self.context, "portal_registry")
        slider_timer = portal_registry[
            "cpskin.core.interfaces.ICPSkinSettings.slider_timer"
        ]
        auto_play_slider = portal_registry[
            "cpskin.core.interfaces.ICPSkinSettings.auto_play_slider"
        ]
        min_items, max_items = self.get_items_number(content)
        slider_type = self.getSliderType(content)
        show_control_nav = DISPLAY_TYPES[slider_type]["control-nav"]
        config = """
        (function($) {
            "use strict";
            var animation = "slide";
            // IE 9 does not support 'slide' animation
            if (navigator.sayswho === 'MSIE 9' || navigator.sayswho === 'IE 9')
            {
            animation = "fade";
            }
            $('.slider_multiple_link').click(function() {
                var href = $(this).attr('href');
                if ($(this).attr('target') == '_blank') window.open(href);
                else window.location.href = href;
            });
            function SlideShow(autoplayslider, content_id) {
                if ($('#slider-' + content_id).parents('.slider-unique-titre').length) {
                    return false;
                }
                else {
                    return autoplayslider;
                }
            }
            function ChangeSlideCurrent(content_id) {
                if ($('#carousel-evenements').parents('.slider-unique-titre').length) {
                    var slides = $('#carousel-evenements').find('.slides');
                    var len = slides.find('li').length;
                    var current = (Math.ceil(len/2) - 1);
                    return current;
                }
            }
            function deplaceSlideCurrent(content_id) {
                $('.slider-unique-titre').each(function (){
                    var carousel = $(this).find('#carousel-%(content_id)s');
                    var carousel_slide = carousel.find('ul.slides');
                    var carousel_slide_active = carousel_slide.find('li.flex-active-slide');
                    var carousel_slide_width = carousel_slide_active.outerWidth(true);
                    var slide_current = carousel_slide_active.index();
                    if (slide_current != 0) {
                        var translate3D_px = -(slide_current * carousel_slide_width) + carousel_slide_width;
                        carousel_slide.css('transform', 'translate3d(' + translate3D_px + 'px, 0px, 0px)');
                    }
                    else {
                        var translate3D_px = carousel_slide_width;
                        carousel_slide.css('transform', 'translate3d(' + translate3D_px + 'px, 0px, 0px)');
                    }
                    carousel_slide.css('transition-duration', '0.2s');
                });
            }
            $('#carousel-%(content_id)s').flexslider({
              animation: animation,
              controlNav: %(show_control_nav)s,
              animationLoop: false,
              slideshow: false,
              itemWidth: 210,
              itemMargin: 5,
              minItems: %(min_items_in_slider)s,
              maxItems: %(max_items_in_slider)s,
              asNavFor: '#slider-%(content_id)s',
              startAt: ChangeSlideCurrent('%(content_id)s'),
              start: function(slider) {
                slider.find('.current-slide').text(slider.currentSlide+1);
                slider.find('.total-slides').text(slider.count);
                deplaceSlideCurrent('%(content_id)s');
              },
              after: function(slider) {
                slider.find('.current-slide').text(slider.currentSlide+1);
                deplaceSlideCurrent('%(content_id)s');
              }
            });
            $('#slider-%(content_id)s').flexslider({
              animation: animation,
              controlNav: %(show_control_nav)s,
              animationLoop: true,
              slideshow: SlideShow(%(auto_play_slider)s, '%(content_id)s'),
              slideshowSpeed: %(slider_timer)s,
              sync: "#carousel-%(content_id)s",
              start: function(slider) {
                  deplaceSlideCurrent('%(content_id)s');
              },
              after: function(slider) {
                deplaceSlideCurrent('%(content_id)s');
              }
            });
         })(jQuery);
        """ % {
            "auto_play_slider": auto_play_slider and "true" or "false",
            "slider_timer": slider_timer,
            "show_control_nav": show_control_nav and "true" or "false",
            "content_id": content_id,
            "min_items_in_slider": min_items,
            "max_items_in_slider": max_items,
        }
        return config

    def scaled_image_url(self, context, obj, isBigImage):
        image = self.collection_image_scale(context, obj)
        return image and image.url or ""

    def collection_image_scale(self, collection, obj):
        scale = getattr(collection, "collection_image_scale", "collection")
        if self.use_new_template(collection):
            return image_scale(obj, "newsImage", scale, generate_tag=False)
        else:
            return image_scale(obj, "newsImage", scale)

    def see_all(self, collection):
        voirlensemble = _(u"Voir l'ensemble des")
        coll_lang = getattr(collection, "language")
        lang = coll_lang if (coll_lang != "" and "-" not in coll_lang) else "fr"  # noqa
        trans = translate(
            voirlensemble, domain=voirlensemble.domain, target_language=lang
        )
        if getattr(collection, "link_text", ""):
            return collection.link_text.encode("utf-8")
        return "{0} {1}".format(trans, collection.Title().lower())

    def get_video(self, video):
        result = utils.embed(video, self.request)
        return result

    def get_class(self, classe):
        if classe == "Media Link":
            return "medialink"
        else:
            return None

    def toLocalizedTime(
        self, time=None, long_format=None, time_only=None, event=None, startend="start"
    ):
        if event:
            if not IEvent.providedBy(event):
                event = event.getObject()
            rs = RecurrenceSupport(event)
            occurences = [occ for occ in rs.occurrences(datetime.today())]
            if len(occurences) >= 1:
                # do not get object which started in the past
                if startend == "start":
                    time = getattr(occurences[0], "start")
                elif startend == "end":
                    time = getattr(occurences[-1], "end")
        return self.context.restrictedTraverse("@@plone").toLocalizedTime(
            time, long_format, time_only
        )

    def get_portal_timezone(self):
        reg = getUtility(IRegistry)
        settings = reg.forInterface(IEventSettings, prefix="plone.app.event")
        return settings.portal_timezone

    def get_event_dates(self, result):
        timezone = self.get_portal_timezone()
        occurences = expand_events([result], RET_MODE_ACCESSORS)
        start_date = getattr(occurences[0], "start")
        end_date = getattr(occurences[-1], "end")

        if not start_date:
            return {"start": "", "end": ""}
        if getattr(start_date, "astimezone", False):
            start_date = start_date.astimezone(pytz.timezone(timezone))
        formated_start = start_date.strftime("%d/%m")
        if not end_date:
            return {"start": formated_start, "end": ""}
        if getattr(end_date, "astimezone", False):
            end_date = end_date.astimezone(pytz.timezone(timezone))
        formated_end = end_date.strftime("%d/%m")
        if formated_start != formated_end:
            return {"start": formated_start, "end": formated_end}
        return {"start": formated_start, "end": ""}

    def is_one_day(self, event):
        occurences = expand_events([event], RET_MODE_ACCESSORS)
        if not occurences:
            start_date = event.start
            end_date = event.end
        else:
            start_date = getattr(occurences[0], "start")
            end_date = getattr(occurences[-1], "end")
        return self.toLocalizedTime(start_date, long_format=0) == self.toLocalizedTime(
            end_date, long_format=0
        )

    def is_with_hours(self, event):
        if getattr(event, "whole_day", False):
            return not (event.whole_day)
        else:
            return (
                self.toLocalizedTime(event.start, long_format=1)[11:] != "00:00"
                or self.toLocalizedTime(event.end, long_format=1)[11:] != "00:00"
            )  # noqa

    def is_open_end(self, event):
        return getattr(event, "open_end", False)

    def see_start_end_date(self, brain, collection):
        if getattr(brain, "start", False) and getattr(brain, "end", False):
            if not getattr(collection, "hide_date", False):
                return True
            else:
                return False
        else:
            return False

    def hide_title(self, collection):
        return getattr(collection, "hide_title", False)

    def hide_see_all_link(self, collection):
        return getattr(collection, "hide_see_all_link", False)

    def hide_date(self, brain, collection):
        """Check if object has a correct effective date.
        If None, you get 01/01/1000 and strftime cannot convert it.
        Also check if collection is checked to see publication date.
        """
        if not getattr(brain, "start", None) and not getattr(
            brain, "end", None
        ):  # noqa
            return getattr(collection, "hide_date", True)
        else:
            # always hide effective date for events
            return True

    def show_day_and_month(self, collection):
        return getattr(collection, "show_day_and_month", False)

    def show_lead_image(self, collection):
        return getattr(collection, "show_lead_image", True)

    def show_descriptions(self, collection):
        return getattr(collection, "show_descriptions", False)

    def use_new_template(self, collection):
        return getattr(collection, "use_new_template", False)

    def use_slider(self, collection):
        display_type = getattr(collection, "display_type", "")
        if not display_type or display_type not in DISPLAY_TYPES:
            return False
        return DISPLAY_TYPES[display_type]["slider"]

    def use_slick(self, collection):
        if self.use_slider(collection) is True:
            return api.portal.get_registry_record(
                "cpskin.core.interfaces.ICPSkinSettings.use_slick",
                default=False,
            )
        return False

    def get_items_number(self, collection):
        display_type = getattr(collection, "display_type", "")
        if display_type != "slider-with-elements-count-choice" and display_type != "slider-slick":
            return 0, 0
        min_items = getattr(collection, "minimum_items_in_slider", 0)
        max_items = getattr(collection, "maximum_items_in_slider", 0)
        return min_items, max_items

    def get_block_class(self, collection):
        display_type = getattr(collection, "display_type", "")
        if not display_type or display_type not in DISPLAY_TYPES:
            return ""
        return DISPLAY_TYPES[display_type]["class"]

    def show_publication_date(self, collection, result):
        if self.hide_date(result, collection):
            return False
        display_type = getattr(collection, "display_type", "")
        return display_type == "slider-with-elements-count-choice"

    def show_event_category_below_image(self, collection):
        return getattr(collection, "show_event_category_below_image", False)


def configure_folderviews(context):
    """
    """
    existingIds = context.objectIds()
    portalPath = api.portal.get().getPhysicalPath()
    contextPath = "/".join(context.getPhysicalPath()[len(portalPath) :])
    if "a-la-une" not in existingIds:
        folder = api.content.create(
            container=context, type="Folder", id="a-la-une", title=_(u"À la une")
        )
        alsoProvides(folder, IFolderViewSelectedContent)
        collection = api.content.create(
            container=folder, type="Collection", id="a-la-une", title=_(u"À la une")
        )
        query = [
            {
                "i": "hiddenTags",
                "o": "plone.app.querystring.operation.selection.is",
                "v": "a-la-une",
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/%s" % contextPath,
            },
        ]
        collection.setQuery(query)
        collection.setSort_on("effective")
        collection.setSort_reversed(True)
        collection.setLayout("summary_view")
        folder.setDefaultPage("a-la-une")
        folder.reindexObject()
    if "actualites" not in existingIds:
        folder = api.content.create(
            container=context, type="Folder", id="actualites", title=_(u"Actualités")
        )
        collection = api.content.create(
            container=folder, type="Collection", id="actualites", title=_(u"Actualités")
        )
        query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": ["News Item"],
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/%s" % contextPath,
            },
        ]
        collection.setQuery(query)
        collection.setSort_on("effective")
        collection.setSort_reversed(True)
        collection.setLayout("summary_view")
        folder.setDefaultPage("actualites")
    else:
        folder = context["actualites"]
    alsoProvides(folder, IFolderViewSelectedContent)
    folder.reindexObject()

    if "evenements" not in existingIds:
        folder = api.content.create(
            container=context, type="Folder", id="evenements", title=_(u"Événements")
        )
        collection = api.content.create(
            container=folder, type="Collection", id="evenements", title=_(u"Événements")
        )
        query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": ["Event"],
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/%s" % contextPath,
            },
        ]
        collection.setQuery(query)
        collection.setSort_on("effective")
        collection.setSort_reversed(True)
        collection.setLayout("summary_view")
        folder.setDefaultPage("evenements")
    else:
        folder = context["evenements"]
    alsoProvides(folder, IFolderViewSelectedContent)
    folder.reindexObject()
    context.setLayout("folderview")



# -*- coding: utf-8 -*-

from Acquisition import aq_base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from plone.app.layout.viewlets.content import ContentRelatedItems
from zope.component import getMultiAdapter


class RightActionsViewlet(ContentRelatedItems):

    index = ViewPageTemplateFile('right_actions.pt')

    def update(self):
        self.has_toc = self.hasTOC()
        try:
            self.has_related = len(self.related_items()) > 0
        except:
            self.has_related = 0
        self.has_portlets = self.hasPortletsToShow()
        self.has_useful_links = self.has_related or self.has_portlets
        self.available = True

    def showPortletsInRightPanel(self):
        portal_registry = api.portal.get_tool('portal_registry')
        return portal_registry['cpskin.core.interfaces.ICPSkinSettings.show_portlets_in_right_actions_panel']  # noqa

    def hasPortletsToShow(self):
        if not self.showPortletsInRightPanel():
            return False
        plone_view = getMultiAdapter((self.context,
                                      self.request),
                                     name=u'plone'
        )
        return plone_view.have_portlets('plone.rightcolumn')

    def hasTOC(self):
        obj = aq_base(self.context)
        getTableContents = getattr(obj, 'getTableContents', None)
        has_toc = False
        if getTableContents is not None:
            try:
                has_toc = getTableContents()
            except KeyError:
                # schema not updated yet
                has_toc = False
        # handle dexterity-behavior
        toc = getattr(obj, 'table_of_contents', None)
        if toc is not None:
            has_toc = toc
        return has_toc



# -*- coding: utf-8 -*-
from Acquisition import aq_get
from collective.dexteritytextindexer.behavior import IDexterityTextIndexer
from cpskin.core.behaviors.booking import IBooking
from cpskin.core.behaviors.directory import ICpskinDirectoryViewSettings
from cpskin.core.behaviors.eventview import ICpskinEventViewSettings
from cpskin.core.behaviors.indexview import ICpskinIndexViewSettings
from cpskin.core.behaviors.metadata import IAdditionalSearchableText
from cpskin.core.behaviors.organization import IOrganizationImages
from cpskin.core.faceted.interfaces import ICPSkinPossibleFacetedNavigable
from cpskin.core.interfaces import IFolderViewSelectedContent
from cpskin.core.setuphandlers import addAutoPlaySliderToRegistry
from cpskin.core.setuphandlers import addCityNameToRegistry
from cpskin.core.setuphandlers import addCollapseMinisiteMenuToRegistry
from cpskin.core.setuphandlers import addFooterSitemapToRegistry
from cpskin.core.setuphandlers import addContentClassesToRegistry
from cpskin.core.setuphandlers import addDescriptionOnThemesOptionToRegistry
from cpskin.core.setuphandlers import addIndexedTaxonomiesToRegistry
from cpskin.core.setuphandlers import addLoadPageMenuToRegistry
from cpskin.core.setuphandlers import addMediaViewletOptionsToRegistry
from cpskin.core.setuphandlers import addPortletsInRightActionsToRegistry
from cpskin.core.setuphandlers import addShowSloganToRegistry
from cpskin.core.setuphandlers import addSliderTimerToRegistry
from cpskin.core.setuphandlers import addSliderTypeToRegistry
from cpskin.core.setuphandlers import addSubMenuPersistenceToRegistry
from cpskin.core.setuphandlers import addTopMenuContentsToRegistry
from cpskin.core.setuphandlers import addTopMenuLeadImageToRegistry
from cpskin.core.setuphandlers import addPersonContactCoreFallbackToRegistry
from cpskin.core.setuphandlers import add_other_xhtml_valid_tags
from cpskin.core.setuphandlers import configCollectiveQucikupload
from cpskin.core.utils import add_behavior
from cpskin.core.utils import remove_behavior
from cpskin.locales import CPSkinMessageFactory as _
from eea.facetednavigation.subtypes.interfaces import IPossibleFacetedNavigable
from plone import api
from plone.app.versioningbehavior.behaviors import IVersionable
from plone.registry import field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from plone.schemaeditor.interfaces import IEditableSchema
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides
from zope.interface import noLongerProvides

import logging


logger = logging.getLogger("cpskin.core")


def remove_slider_type_to_registry(context):
    registry = getUtility(IRegistry)
    records = registry.records
    key = "cpskin.core.interfaces.ICPSkinSettings.slider_type"
    if key in records:
        logger.info(
            "Remove cpskin.core.interfaces.ICPSkinSettings.slider_type from registry"
        )  # noqa
        records.__delitem__(key)


def install_imiogdpr(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runAllImportStepsFromProfile("profile-imio.gdpr:default")


def migrate_hide_title_for_sliders(context):
    index_collections_brains = api.content.find(
        portal_type="Collection",
        object_provides=ICpskinIndexViewSettings.__identifier__,
    )
    for brain in index_collections_brains:
        obj = brain.getObject()
        display_type = getattr(obj, "display_type") or ""
        if "slider" in display_type:
            obj.hide_title = True


def split_show_day_and_month(context):
    index_collections_brains = api.content.find(
        portal_type="Collection",
        object_provides=ICpskinIndexViewSettings.__identifier__,
    )
    for brain in index_collections_brains:
        obj = brain.getObject()
        show_day_and_month = getattr(obj, "show_day_and_month")
        obj.show_lead_image = not (show_day_and_month)


def upgrade_js_registry(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "jsregistry")


def install_fancybox(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runAllImportStepsFromProfile("profile-collective.js.fancybox:default")


def upgrade_organization_gallery(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "cssregistry")
    context.runImportStepFromProfile("profile-cpskin.core:default", "jsregistry")
    context.runImportStepFromProfile("profile-cpskin.core:default", "viewlets")


def add_images_behavior(context):
    add_behavior("organization", IOrganizationImages.__identifier__)


def enable_directory_versioning(context):
    add_behavior('directory', IVersionable.__identifier__)
    add_behavior('organization', IVersionable.__identifier__)
    add_behavior('person', IVersionable.__identifier__)
    add_behavior('position', IVersionable.__identifier__)
    context.runImportStepFromProfile("profile-cpskin.core:default", "repositorytool")
    context.runImportStepFromProfile("profile-cpskin.core:default", "difftool")


def set_other_xhtml_valid_tags(context):
    add_other_xhtml_valid_tags()


def use_sc_social_like_instead_of_bookmarks(context):
    portal_setup = api.portal.get_tool("portal_setup")
    portal_setup.runAllImportStepsFromProfile("profile-sc.social.like:default")
    from sc.social.like.interfaces import ISocialLikeSettings

    api.portal.set_registry_record(
        "plugins_enabled", ("Facebook",), interface=ISocialLikeSettings
    )
    api.portal.set_registry_record(
        "sc.social.like.interfaces.ISocialLikeSettings.fbbuttons", (u"Share",)
    )
    api.portal.set_registry_record(
        "sc.social.like.interfaces.ISocialLikeSettings.fbshowlikes", False
    )
    portal_properties = api.portal.get_tool("portal_properties")
    sc_social_bookmarks_properties = portal_properties.get(
        "sc_social_bookmarks_properties"
    )

    if sc_social_bookmarks_properties:
        enabled_portal_types = sc_social_bookmarks_properties.enabled_portal_types

        api.portal.set_registry_record(
            "enabled_portal_types", enabled_portal_types, interface=ISocialLikeSettings
        )
        bookmark_providers = sc_social_bookmarks_properties.bookmark_providers
        if "Google Bookmarks" in bookmark_providers:
            lst = list(bookmark_providers)
            index = lst.index("Google Bookmarks")
            lst[index] = "Google+"
            bookmark_providers = tuple(lst)
        api.portal.set_registry_record(
            "plugins_enabled", bookmark_providers, interface=ISocialLikeSettings
        )
        # clean up
        portal_properties.manage_delObjects(sc_social_bookmarks_properties.id)
        portal_setup.runAllImportStepsFromProfile(
            "profile-sc.social.bookmarks:uninstall"
        )


def upgrade_registry_for_themes_descriptions(context):
    addDescriptionOnThemesOptionToRegistry()


def upgrade_registry_for_media_viewlet(context):
    addMediaViewletOptionsToRegistry(upgrade=True)


def add_right_actions(context):
    addPortletsInRightActionsToRegistry()
    context.runImportStepFromProfile("profile-cpskin.core:default", "viewlets")
    context.runImportStepFromProfile("profile-cpskin.core:default", "jsregistry")


def upgrade_registry_for_slogan(context):
    addShowSloganToRegistry()


def upgrade_registry_for_minisite_menu(context):
    addCollapseMinisiteMenuToRegistry()


def upgrade_registry_for_show_footer_sitemap(context):
    addFooterSitemapToRegistry()


def upgrade_registry_for_content_classes(context):
    addContentClassesToRegistry()


def update_types(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "typeinfo")


def clean_portal_setup(context):
    """
    Force uninstall of packages that are not marked as installed but were
    imported in portal_setup : #17714
    """
    ps = api.portal.get_tool("portal_setup")
    qi = api.portal.get_tool("portal_quickinstaller")
    quick_installed = [p["id"] for p in qi.listInstalledProducts()]
    all_profiles = [
        p["id"] for p in context.listContextInfos() if p["type"] == "extension"
    ]
    cpskin_profiles = [
        p for p in all_profiles if p.startswith("profile-cpskin.diazotheme.")
    ]
    for profile_id in cpskin_profiles:
        if "uninstall" in profile_id:
            continue
        if ps.getLastVersionForProfile(profile_id) == "unknown":
            # profile is not installed
            logger.info("{0} is not installed - skipping".format(profile_id))
            continue
        package_id = profile_id.split("-")[1].split(":")[0]
        if package_id in quick_installed:
            # profile is installed, product is also installed in quickinstaller
            logger.info("{0} is well installed - skipping".format(profile_id))
            continue
        logger.warn("{0} is NOT installed correctly".format(profile_id))
        uninstall_profile_id = "profile-{0}:uninstall".format(package_id)
        if uninstall_profile_id in all_profiles:
            # install lesscss because uninstall profiles need it.
            if not qi.isProductInstalled("collective.lesscss"):
                context.runAllImportStepsFromProfile(
                    "profile-collective.lesscss:default"
                )
                qi.installProduct("collective.lesscss")
            context.runAllImportStepsFromProfile(uninstall_profile_id)
            ps.unsetLastVersionForProfile(profile_id)
            qi.uninstallProducts(products=[str(package_id)])
            logger.info("{0} uninstalled successfully !".format(package_id))
        else:
            logger.warn("No uninstall profile for {0}".format(package_id))

    psk = api.portal.get_tool("portal_skins")
    selected_skin = "Sunburst Theme"
    if psk.default_skin != selected_skin:
        psk.default_skin = selected_skin
        request = aq_get(context, "REQUEST", None)
        portal = api.portal.get()
        portal.changeSkin(selected_skin, request)
        logger.info("Restored default_skin : {0}".format(selected_skin))


def upgrade_viewlets(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "viewlets")


def upgrade_registry_for_top_menu(context):
    addTopMenuContentsToRegistry()
    addTopMenuLeadImageToRegistry()


def upgrade_registry_for_contact_core_fallback(context):
    addPersonContactCoreFallbackToRegistry()


def migrate_a_la_une_sliders(context):
    brains = api.content.find(id="a-la-une", portal_type="Collection")
    for brain in brains:
        obj = brain.getObject()
        obj.display_type = u"slider-with-carousel"


def update_theme_variables(context):
    context.runAllImportStepsFromProfile("profile-cpskin.theme:default")
    key = "plone.app.theming.interfaces.IThemeSettings.parameterExpressions"
    params = {}
    params["globalnavsetting"] = "python: 'always'"
    params["isinminisitemode"] = "context/@@isInMinisiteMode"
    params["is_homepage"] = "context/@@is_homepage"
    params["environment"] = "context/@@environment"
    params["login_message"] = "context/@@get_login_message"
    api.portal.set_registry_record(key, params)


def empty_value_of_link_text(context):
    for brain in api.content.find(portal_type="Collection"):
        obj = brain.getObject()
        path = "/".join(obj.getPhysicalPath())
        logger.info("set link_text empty for {0}".format(path))
        setattr(obj, "link_text", "")


def upgrade_to_nineteen(context):
    context.runImportStepFromProfile("profile-cpskin.core:to19", "jsregistry")
    pc = api.portal.get_tool("portal_catalog")
    for brain in pc.unrestrictedSearchResults(
        object_provides=IPossibleFacetedNavigable.__identifier__
    ):
        obj = brain.getObject()
        if obj:
            alsoProvides(obj, ICPSkinPossibleFacetedNavigable)


def clean_old_keyword_homepage(context):
    behavior_name = "cpskin.core.behaviors.metadata.IUseKeywordHomepage"
    types = [
        "Folder",
        "Collection",
        "Document",
        "Event",
        "News Item",
        "organization",
        "person",
    ]
    for type_name in types:
        remove_behavior(type_name, behavior_name)


def move_cpskin_actions(context):
    context.runImportStepFromProfile("profile-cpskin.core:to17", "actions")
    context.runImportStepFromProfile("profile-cpskin.core:default", "actions")


def add_navigation_toggle_action(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "actions")


def add_directory_view_behavior(context):
    add_behavior("directory", ICpskinDirectoryViewSettings.__identifier__)


def add_index_view_behavior(context):
    registry = getUtility(IRegistry)
    del registry.records["cpskin.core.interfaces.ICPSkinSettings.homepage_keywords"]
    add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)


def add_banner_view_behavior(context):
    add_behavior("Event", ICpskinEventViewSettings.__identifier__)


def upgrade_homepage_keywords(context):
    # add_homepage_keywords()
    pass


def upgrade_minisite_menu(context):
    # add new viewlet cpskin.minisite
    context.runImportStepFromProfile("profile-cpskin.core:default", "viewlets")
    context.runImportStepFromProfile("profile-cpskin.menu:default", "jsregistry")
    context.runImportStepFromProfile("profile-cpskin.minisite:default", "actions")


def upgrade_city_name(context):
    addCityNameToRegistry()


def upgrade_slider_type(context):
    addSliderTypeToRegistry()


def upgrade_indexed_taxonomies(context):
    addIndexedTaxonomiesToRegistry()


def upgrade_search_position(context):
    registry = getUtility(IRegistry)
    records = registry.records
    if "cpskin.core.interfaces.ICPSkinSettings.search_position" in records:
        return

    logger.info(
        "Adding cpskin.core.interfaces.ICPSkinSettings.search_position to registry"
    )  # noqa
    record = Record(
        field.TextLine(
            title=_(u"Search position"),
            description=_(u"Search box position in eligible themes."),
            required=True,
            default=u"default_position",
        ),
        value=u"default_position",
    )
    records["cpskin.core.interfaces.ICPSkinSettings.search_position"] = record


def upgrade_footer_viewlet(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "viewlets")


def upgrade_to_eleven(context):
    addSubMenuPersistenceToRegistry()


def upgrade_to_eight(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "rolemap")


def upgrade_to_seven(context):
    addAutoPlaySliderToRegistry()
    addSliderTimerToRegistry()


def upgrade_to_six(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "rolemap")
    context.runImportStepFromProfile("profile-cpskin.core:default", "sharing")
    portal = api.portal.get()
    portal.manage_permission(
        "CPSkin: Edit keywords",
        roles=["Portlets Manager", "Manager", "Site Administrator"],
        acquire=True,
    )
    site_properties = api.portal.get_tool("portal_properties").site_properties
    site_properties.allowRolesToAddKeywords = (
        "Manager",
        "Site Administrator",
        "Portlets Manager",
    )


def upgrade_to_five(context):
    interfaces = [
        "cpskin.core.viewlets.interfaces.IViewletMenuToolsBox",
        "cpskin.core.viewlets.interfaces.IViewletMenuToolsFaceted",
    ]
    catalog = api.portal.get_tool("portal_catalog")
    for interface in interfaces:
        brains = catalog({"object_provides": interface})
        for brain in brains:
            obj = brain.getObject()
            provided = directlyProvidedBy(obj)
            cleanedProvided = [i for i in provided if i.__identifier__ != interface]
            directlyProvides(obj, cleanedProvided)  # noqa
            obj.reindexObject()

    portal_javascripts = api.portal.get_tool("portal_javascripts")
    portal_javascripts.unregisterResource("++resource++cpskin.core.menutools.js")


def upgrade_to_four(context):
    addLoadPageMenuToRegistry()


def upgrade_to_three(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "actions")
    context.runImportStepFromProfile("profile-cpskin.core:default", "rolemap")
    context.runImportStepFromProfile("profile-cpskin.core:default", "sharing")
    portal = api.portal.get()
    portal.manage_permission(
        "Portlets: Manage portlets",
        roles=["Editor", "Portlets Manager", "Manager", "Site Administrator"],
        acquire=True,
    )


def upgrade_to_two(context):
    context.runAllImportStepsFromProfile("profile-cpskin.policy:default")
    portal_catalog = api.portal.get_tool("portal_catalog")
    portal_atct = api.portal.get_tool("portal_atct")
    attrs = ("IAmTags", "HiddenTags", "ISearchTags")
    for attr in attrs:
        if attr in portal_catalog.indexes():
            portal_catalog.delIndex(attr)
        if attr in portal_atct.topic_indexes:
            portal_atct.removeIndex(attr)
    for brain in portal_catalog.searchResults():  # noqa
        obj = brain.getObject()
        for attr in attrs:
            if hasattr(obj, attr):
                try:
                    delattr(obj, attr)
                except AttributeError:
                    logger.info("No {0} on: {1}".format(attr, obj))


def upgrade_add_booking_behavior(context):
    portal_types = api.portal.get_tool(name="portal_types")
    event_type = portal_types.get("Event")
    fields = event_type.lookupSchema().names()

    if "tarifs" not in fields and "reservation" not in fields:
        # This upgrade step is specific to Liege, where those fields have been
        # added TTW.
        return

    add_behavior("Event", IBooking.__identifier__)
    for brain in api.content.find(portal_type="Event"):
        obj = brain.getObject()
        if getattr(obj, "tarifs") is not None:
            obj.booking_price = getattr(obj, "tarifs")
        if getattr(obj, "reservation") is not None:
            has_booking = getattr(obj, "reservation")
            if has_booking:
                obj.booking_type = "mandatory"
            else:
                obj.booking_type = "no_booking"
    schema = IEditableSchema(event_type.lookupSchema())
    schema.removeField("tarifs")
    schema.removeField("reservation")


def upgrade_add_specific_typesUseViewActionInListings(context):
    site_properties = api.portal.get_tool("portal_properties").site_properties
    site_properties.typesUseViewActionInListings = ("File", "Image")


def upgrade_add_default_page_types_document(context):
    site_properties = api.portal.get_tool("portal_properties").site_properties
    site_properties.default_page_types = ("Topic", "FormFolder", "Document")


def upgrade_limit_plone_site_portal_type(context):
    portal_types = api.portal.get_tool("portal_types")
    types = ["Plone Site", "LRF"]
    for t in types:
        plone_type = portal_types.get(t)
        if plone_type:
            plone_type.filter_content_types = True
            plone_type.allowed_content_types = ("Document", "Folder", "Image")


def upgrade_footer_minisite(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "viewlets")
    brains = api.content.find(id="footer-mini-site")
    for b in brains:
        obj = b.getObject()
        noLongerProvides(obj, IFolderViewSelectedContent)
        obj.reindexObject()
        logger.info("Removed %s from minisite index view" % obj.absolute_url())


def upgrade_css_js_registry(context):
    context.runImportStepFromProfile("profile-cpskin.core:default", "cssregistry")
    context.runImportStepFromProfile("profile-cpskin.core:default", "jsregistry")


def set_quickupload_properties(context):
    configCollectiveQucikupload()


def add_searchable_on_organization(context):
    add_behavior("organization", IAdditionalSearchableText.__identifier__)
    add_behavior("organization", IDexterityTextIndexer.__identifier__)


def to_65_use_slick(context):
    registry = getUtility(IRegistry)
    records = registry.records

    if "cpskin.core.interfaces.ICPSkinSettings.use_slick" in records:
        return

    logger.info(
        "Adding cpskin.core.interfaces.ICPSkinSettings.use_slick to registry")
    record = Record(
        field.Bool(
            title=_(u"Use slick for slider"),
            description=_(u"Do you want to use slick instead of flexslider ?"),
            required=False,
            default=False,
        ),
        value=False,
    )
    records["cpskin.core.interfaces.ICPSkinSettings.use_slick"] = record


def upgrade_limit_plone_site_portal_type_2(context):
    portal_types = api.portal.get_tool("portal_types")
    types = ["Plone Site", "LRF"]
    for t in types:
        plone_type = portal_types.get(t)
        if plone_type:
            plone_type.filter_content_types = True
            plone_type.allowed_content_types = ("Document", "Folder", "Image", "Link")



# -*- coding: utf-8 -*-
from collective.geo.behaviour.interfaces import ICoordinates
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from cpskin.core.utils import add_keyword
from cpskin.core.utils import add_leadimage_from_file
from imio.gdpr.interfaces import IGDPRSettings
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.Five.browser import BrowserView
from z3c.relationfield.relation import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.event import notify
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent
from zope.viewlet.interfaces import IViewletManager

import json
import os
import unittest


class TestViewlets(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.folder = api.content.create(self.portal, 'Folder', 'folder')

    def test_navigation_toggle_viewlet(self):
        folder = api.content.create(
            container=self.portal, type='Folder', id='folder1')
        view = getMultiAdapter(
            (folder, self.request), name='navigation_toggle_activation')
        navtogglesettings = view.settings
        self.assertEqual(navtogglesettings.selectors, ())
        self.assertFalse(view.is_enabled)
        self.assertTrue(view.can_enable_navigation_toggle)
        self.assertFalse(view.can_disable_navigation_toggle)

        view.enable_navigation_toggle()
        self.assertTrue(view.is_enabled)
        self.assertFalse(view.can_enable_navigation_toggle)
        self.assertTrue(view.can_disable_navigation_toggle)
        self.assertEqual(navtogglesettings.selectors, (u'/folder1', ))

        view.disable_navigation_toggle()
        self.assertFalse(view.is_enabled)
        self.assertTrue(view.can_enable_navigation_toggle)
        self.assertFalse(view.can_disable_navigation_toggle)
        self.assertEqual(navtogglesettings.selectors, ())

    def test_set_media_viewlet(self):
        view = getMultiAdapter(
            (self.portal, self.request), name='media_activation')
        self.assertFalse('videos' in self.portal.keys())
        self.assertFalse(view.is_enabled)
        self.assertFalse(view.can_disable_media)

        view.enable_media()
        self.assertTrue(view.is_enabled)
        self.assertTrue(view.can_disable_media)
        self.assertTrue('videos' in self.portal.keys())

        view.disable_media()
        self.assertFalse(view.is_enabled)
        self.assertFalse(view.can_disable_media)
        self.assertTrue('videos' in self.portal.keys())

        view.enable_media()
        self.assertTrue(view.is_enabled)
        self.assertTrue(view.can_disable_media)
        self.assertTrue('videos' in self.portal.keys())

    def test_media_viewlet(self):
        keywords = ['album-a-la-une']
        view = getMultiAdapter(
            (self.portal, self.request), name='media_activation')
        # create video collection used to viewlet
        view.enable_media()
        album_collection = self.portal.albums.index
        query = [
            {
                'i': 'hiddenTags',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': keywords
            },
        ]

        # Set hiddenTags and leadimage behaviors
        add_behavior('Folder', 'cpskin.core.behaviors.metadata.IHiddenTags')
        add_behavior(
            'Folder',
            'plone.app.contenttypes.behaviors.leadimage.ILeadImage')

        album_collection.query = query
        album = api.content.create(
            container=self.portal, type='Folder', id='testalbum')

        # getting viewlet
        view = BrowserView(self.portal, self.request)
        manager_name = 'plone.belowcontent'
        manager = queryMultiAdapter(
            (self.portal, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.media']
        self.assertEqual(len(my_viewlet), 1)
        media_viewlet = my_viewlet[0]

        self.assertTrue(media_viewlet.available())
        self.assertEqual(
            media_viewlet.get_albums_collection().getPhysicalPath(),
            self.portal.albums.index.getPhysicalPath())
        self.assertEqual(len(media_viewlet.get_albums()), 0)

        # Set lead image to album folder
        add_leadimage_from_file(album, 'cpskinlogo.png')
        add_keyword(album, 'hiddenTags', keywords)
        self.assertEqual(len(media_viewlet.get_albums()), 1)

    def test_above_related_contacts_viewlet(self):
        add_behavior(
            'Event', 'cpskin.core.behaviors.metadata.IRelatedContacts')
        event = api.content.create(
            container=self.folder,
            type='Event',
            id='myevent'
        )

        # getting viewlet
        view = BrowserView(event, self.request)
        manager_name = 'plone.abovecontentbody'
        manager = queryMultiAdapter(
            (event, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.above_related_contacts']  # noqa
        self.assertEqual(len(my_viewlet), 1)
        above_viewlet = my_viewlet[0]

        contacts = above_viewlet.get_contacts()
        self.assertEqual(contacts, [])
        self.assertFalse(above_viewlet.available())

        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        person = api.content.create(
            container=directory, type='person', id='person')
        person.firstname = u'Foo'
        person.lastname = u'Bar'
        person.gender = u'F'
        person.street = u'Zoning Industriel'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'

        # set related contact
        intids = getUtility(IIntIds)
        to_id = intids.getId(person)
        rv = RelationValue(to_id)
        event.aboveContentContact = event.aboveContentContact + [rv]
        self.assertTrue(above_viewlet.available())
        contacts = above_viewlet.get_contacts()
        self.assertEqual(len(contacts), 1)
        self.assertEqual(contacts[0], person)

        self.assertIn('Mornimont', above_viewlet.render())
        self.assertNotIn('Foo Bar', above_viewlet.render())

        event.aboveVisbileFields = ('street', 'number', 'zip_code', 'city')
        self.assertIn('5190', above_viewlet.render())
        self.assertIn('Mornimont', above_viewlet.render())

        event.aboveVisbileFields = ('firstname',)
        self.assertIn('Foo', above_viewlet.render())

        # test get_title method
        self.assertFalse(above_viewlet.get_title(person))
        event.aboveVisbileFields = ('title')
        self.assertEqual(
            above_viewlet.get_title(person),
            u'<span class="related-contact-title">Foo Bar</span>')

    def test_below_related_contacts_viewlet(self):
        add_behavior(
            'Event', 'cpskin.core.behaviors.metadata.IRelatedContacts')
        event = api.content.create(
            container=self.folder,
            type='Event',
            id='myevent'
        )

        # getting viewlet
        view = BrowserView(event, self.request)
        manager_name = 'plone.belowcontentbody'
        manager = queryMultiAdapter(
            (event, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.below_related_contacts']  # noqa
        self.assertEqual(len(my_viewlet), 1)
        below_viewlet = my_viewlet[0]

        contacts = below_viewlet.get_contacts()
        self.assertEqual(contacts, [])
        self.assertFalse(below_viewlet.available())

        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        person = api.content.create(
            container=directory, type='person', id='person')
        person.firstname = u'Foo'
        person.lastname = u'Bar'
        person.gender = u'F'
        person.street = u'Zoning Industriel'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'

        # set related contact
        intids = getUtility(IIntIds)
        to_id = intids.getId(person)
        rv = RelationValue(to_id)
        event.belowContentContact = event.belowContentContact + [rv]
        self.assertTrue(below_viewlet.available())
        contacts = below_viewlet.get_contacts()
        self.assertEqual(len(contacts), 1)
        self.assertEqual(contacts[0], person)

        self.assertNotIn('Mornimont', below_viewlet.render())
        self.assertIn('Foo Bar', below_viewlet.render())

        event.belowVisbileFields = ('zip_code',)
        self.assertNotIn('5190', below_viewlet.render())
        self.assertNotIn('Foo', below_viewlet.render())

        # test get_title method
        self.assertFalse(below_viewlet.get_title(person))
        event.belowVisbileFields = ('title')
        self.assertEqual(
            below_viewlet.get_title(person),
            u'<a href="http://nohost/plone/directory/person" target="_blank"><h4>Foo Bar</h4></a>')  # noqa

    def test_use_parent_address(self):
        add_behavior(
            'Event', 'cpskin.core.behaviors.metadata.IRelatedContacts')
        event = api.content.create(
            container=self.folder,
            type='Event',
            id='myevent'
        )

        # getting viewlet
        view = BrowserView(event, self.request)
        manager_name = 'plone.belowcontentbody'
        manager = queryMultiAdapter(
            (event, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.below_related_contacts']  # noqa
        self.assertEqual(len(my_viewlet), 1)
        below_viewlet = my_viewlet[0]

        contacts = below_viewlet.get_contacts()
        self.assertEqual(contacts, [])
        self.assertFalse(below_viewlet.available())

        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        person = api.content.create(
            container=directory, type='person', id='person')
        person.firstname = u'Foo'
        person.lastname = u'Bar'
        person.gender = u'F'
        person.street = u'Zoning Industriel'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'
        person.use_parent_address = True

        # set related contact
        intids = getUtility(IIntIds)
        to_id = intids.getId(person)
        rv = RelationValue(to_id)
        event.belowContentContact = event.belowContentContact + [rv]
        self.assertTrue(below_viewlet.available())
        contacts = below_viewlet.get_contacts()
        self.assertEqual(len(contacts), 1)
        self.assertEqual(contacts[0], person)

        self.assertNotIn('Mornimont', below_viewlet.render())
        self.assertIn('Foo Bar', below_viewlet.render())

        event.belowVisbileFields = ('zip_code',)
        self.assertNotIn('5190', below_viewlet.render())
        self.assertNotIn('Foo', below_viewlet.render())

    def test_use_parent_address_in_map(self):
        add_behavior(
            'Event', 'cpskin.core.behaviors.metadata.IRelatedContacts')
        event = api.content.create(
            container=self.folder,
            type='Event',
            id='myevent'
        )

        # getting viewlet
        view = BrowserView(event, self.request)
        manager_name = 'plone.belowcontentbody'
        manager = queryMultiAdapter(
            (event, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.related_contacts_map']  # noqa
        self.assertEqual(len(my_viewlet), 1)
        map_viewlet = my_viewlet[0]

        contacts = map_viewlet.get_contacts()
        self.assertEqual(contacts, [])
        self.assertFalse(map_viewlet.available())

        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')

        organization = api.content.create(
            container=directory, type='organization', id='organization')
        organization.title = u'IMIO'
        organization.street = u'Rue Léon Morel'
        organization.number = u'1'
        organization.zip_code = u'5032'
        organization.city = u'Isnes'
        organization.use_parent_address = False

        orga_child = api.content.create(
            container=organization, type='organization', id='organization')
        orga_child.title = 'DevOps'
        orga_child.use_parent_address = False

        # set related contact
        intids = getUtility(IIntIds)
        to_id = intids.getId(orga_child)
        rv = RelationValue(to_id)
        event.belowContentContact = [rv]
        add_behavior('organization', ICoordinates.__identifier__)
        notify(ObjectModifiedEvent(organization))

        self.assertTrue(map_viewlet.see_map_link(organization))
        self.assertFalse(map_viewlet.see_map_link(orga_child))
        data_geojson = map_viewlet.data_geojson()
        results = json.loads(data_geojson)
        self.assertEqual(len(results['features']), 0)

        orga_child.use_parent_address = True
        notify(ObjectModifiedEvent(orga_child))
        self.assertTrue(map_viewlet.see_map_link(orga_child))
        data_geojson = map_viewlet.data_geojson()
        results = json.loads(data_geojson)
        self.assertEqual(
            results['features'][0]['properties']['address'],
            u'Rue L\xe9on Morel, 1<br />5032 Isnes'
        )

    def test_related_contacts_map_viewlet(self):
        add_behavior(
            'Event', 'cpskin.core.behaviors.metadata.IRelatedContacts')

        event = api.content.create(
            container=self.folder,
            type='Event',
            id='myevent'
        )

        # getting viewlet
        view = BrowserView(event, self.request)
        manager_name = 'plone.belowcontentbody'
        manager = queryMultiAdapter(
            (event, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.related_contacts_map']  # noqa
        self.assertEqual(len(my_viewlet), 1)
        map_viewlet = my_viewlet[0]

        contacts = map_viewlet.get_contacts()
        self.assertEqual(contacts, [])
        self.assertFalse(map_viewlet.available())

        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        add_behavior('person', ICoordinates.__identifier__)
        from collective.contact.core.behaviors import IContactDetails
        add_behavior('person', IContactDetails.__identifier__)
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        person = api.content.create(
            container=directory, type='person', id='person')
        person.firstname = u'Foo'
        person.lastname = u'Bar'
        person.gender = u'F'
        person.street = u'Rue de la Vieille Sambre'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'
        person.use_parent_address = False
        notify(ObjectModifiedEvent(person))
        person.street = u'Zoning Industriel'
        person.number = u'34'
        person.zip_code = u'5190'
        person.city = u'Mornimont'
        # set related contact
        intids = getUtility(IIntIds)
        to_id = intids.getId(person)
        rv = RelationValue(to_id)
        event.belowContentContact = event.belowContentContact + [rv]
        self.assertTrue(map_viewlet.available())
        contacts = map_viewlet.get_contacts()
        self.assertEqual(len(contacts), 1)
        self.assertEqual(contacts[0], person)

        # Do not see duplicate
        event.aboveContentContact = event.aboveContentContact + [rv]
        contacts = map_viewlet.get_contacts()
        self.assertEqual(len(contacts), 1)
        self.assertEqual(contacts[0], person)

        person2 = api.content.create(directory, 'person', 'person2')
        person2.firstname = u'James'
        person2.lastname = u'Bond'
        person2.gender = u'M'
        person2.street = u"Boulevard d'avroy"
        person2.number = u'007'
        person2.zip_code = u'4000'
        person2.city = u'Liège'
        notify(ObjectModifiedEvent(person2))
        person2.street = u"Boulevard d'avroy"
        person2.number = u'007'
        person2.zip_code = u'4000'
        person2.city = u'Liège'
        person2.use_parent_address = False
        to_id2 = intids.getId(person2)
        rv2 = RelationValue(to_id2)
        event.aboveContentContact = event.aboveContentContact + [rv2]
        contacts = map_viewlet.get_contacts()
        self.assertEqual(len(contacts), 2)
        event.see_map = False
        self.assertFalse(map_viewlet.available())
        event.see_map = True

        geojson = map_viewlet.data_geojson()
        results = json.loads(geojson)
        self.assertEqual(results['type'], u'FeatureCollection')
        self.assertEqual(len(results['features']), 2)
        self.assertEqual(u'', results['features'][0]['properties']['image'])

        # add a logo
        core_path = '/'.join(os.path.dirname(__file__).split('/')[:-1])
        data_path = os.path.join(core_path, 'data')
        file_path = os.path.join(data_path, 'cpskinlogo.png')
        from plone.namedfile.file import NamedBlobImage
        namedblobimage = NamedBlobImage(
            data=open(file_path, 'r').read(),
            filename=unicode('cpskinlogo.png')
        )
        person.logo = namedblobimage
        geojson = map_viewlet.data_geojson()
        results = json.loads(geojson)
        self.assertEqual(
            u'http://nohost/plone/directory/person/@@images/logo/thumb',
            results['features'][0]['properties']['image'])

    def test_footer_viewlet(self):
        # getting viewlet
        view = BrowserView(self.portal, self.request)
        manager_name = 'plone.portalfooter'
        manager = queryMultiAdapter(
            (self.portal, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == 'cpskin.footer']  # noqa
        self.assertEqual(len(my_viewlet), 1)
        viewlet = my_viewlet[0]
        self.assertNotIn('Legal notice', viewlet.render())
        api.portal.set_registry_record(
            'is_text_ready',
            True,
            interface=IGDPRSettings
        )
        self.assertIn('Legal notice', viewlet.render())



# -*- coding: utf-8 -*-
from DateTime import DateTime
from collective.anysurfer.layout import LayoutPolicy as AnysurferLayoutPolicy
from eea.facetednavigation.layout.interfaces import IFacetedLayout
from eea.facetednavigation.subtypes.interfaces import IFacetedNavigable
from plone import api
from plone.app.layout.globals import layout as base
from plone.app.layout.globals.interfaces import ILayoutPolicy
from plone.i18n.normalizer import IIDNormalizer
from zope.component import queryAdapter
from zope.component import queryUtility
from zope.interface import implements

from cpskin.citizen.utils import is_citizen


class LayoutPolicy(AnysurferLayoutPolicy, base.LayoutPolicy):
    """
    Enhanced layout policy
    """
    implements(ILayoutPolicy)

    def bodyClass(self, template, view):
        """
        Extend Plone to add a CSS class on <body> based on :
        1. the 1st level folder of the current context :
         - 'section-theme1' or 'section-theme2' or ... for portal tabs folders
           based on their positions
         - section-notheme if not in a portal tab folder
        2. the citizen user
        3. minisite
        4. homepage
        5. the portal_type of the collection (if any)
        6. the expiration of the content
        7. the layout used for faceted navigations
        """
        context = self.context

        # Get default body classes
        body_class = base.LayoutPolicy.bodyClass(self, template, view)

        # Get 1st level folders appearing in navigation
        portal_catalog = api.portal.get_tool(name='portal_catalog')
        navtreeProps = api.portal.get_tool(name='portal_properties').navtree_properties
        navigation_root = api.portal.get_navigation_root(context)
        queryDict = {}
        queryDict['path'] = {'query': '/'.join(navigation_root.getPhysicalPath()), 'depth': 1}
        if navtreeProps.enable_wf_state_filtering:
            queryDict['review_state'] = navtreeProps.wf_states_to_show
        queryDict['sort_on'] = 'getObjPositionInParent'
        queryDict['portal_type'] = 'Folder'
        queryDict['is_default_page'] = False
        brains = portal_catalog(queryDict)
        res = [b for b in brains if b.id not in navtreeProps.idsNotToList]

        # Get the first level of the current
        actual_url_path = '/'.join(context.getPhysicalPath())
        # Check if we are in a theme and check if we are in the right one (position)
        index = 1
        inTheme = False
        for brain in res:
            # checking startswith is not enough
            # see ticket #1227 :
            # if theme1 id is "theme" and theme2 id is "theme2", while being in the
            # theme2, it starts with 'theme' so it returns True to checking if being in theme 1...
            brainPath = brain.getPath()
            if actual_url_path.startswith(brainPath):
                brainPathLen = len(brainPath)
                if len(actual_url_path) == brainPathLen \
                   or actual_url_path[brainPathLen:brainPathLen + 1] == '/':
                    inTheme = True
                    body_class += ' section-theme%s' % index
            index += 1

        if not inTheme:
            body_class += ' section-notheme'

        header_class = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.header_class'
        )
        if header_class:
            body_class += ' {0}'.format(header_class)

        navigation_class = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.navigation_class'
        )
        if navigation_class:
            body_class += ' {0}'.format(navigation_class)

        content_columns_class = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.columns_class'
        )
        if content_columns_class:
            body_class += ' {0}'.format(content_columns_class)

        footer_class = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.footer_class'
        )
        if footer_class:
            body_class += ' {0}'.format(footer_class)

        minisite = self.request.get('cpskin_minisite', None)
        if minisite:
            if minisite.is_in_minisite_mode:
                body_class += ' in-minisite in-minisite-out-portal'
            elif minisite.is_in_portal_mode:
                body_class += ' in-minisite in-minisite-in-portal'

        if is_main_homepage(context):
            body_class += ' main-homepage'

        user = api.user.get_current()
        if is_citizen(user):
            body_class += ' user-citizen'

        if context.portal_type == 'Collection':
            normalizer = queryUtility(IIDNormalizer).normalize
            query = context.query
            portal_types = []
            if query is not None:
                for criteria in query:
                    if criteria.get('i') == 'portal_type':
                        portal_types = criteria.get('v')
            for portal_type in portal_types:
                body_class += ' collection-%s' % normalizer(portal_type)

        expiration_date = context.expiration_date
        if expiration_date and expiration_date < DateTime():
            body_class += ' expired-content'

        if IFacetedNavigable.providedBy(context):
            faceted_adapter = queryAdapter(context, IFacetedLayout)
            if faceted_adapter:
                body_class += ' %s' % faceted_adapter.layout

        return body_class


def is_main_homepage(context):
    homepage_types = ['LRF', 'Plone Site']
    if context.portal_type in homepage_types:
        return True
    return False



# -*- coding: utf-8 -*-
from collective.documentgenerator.browser.generation_view import DocumentGenerationView  # noqa
from collective.documentgenerator.helper.dexterity import DXDocumentGenerationHelperView  # noqa
from collective.taxonomy.interfaces import ITaxonomy
from cpskin.core.interfaces import IFolderViewSelectedContent as IFVSC
from cpskin.core.utils import format_phone
from cpskin.core.utils import safe_utf8
from cpskin.core.utils import safe_unicode
from cpskin.locales import CPSkinMessageFactory as _
from DateTime import DateTime
from eea.facetednavigation.interfaces import IFacetedNavigable
from imio.dashboard.utils import getDashboardQueryResult
from plone import api
from plone.app.event.base import date_speller
from plone.app.event.base import dates_for_display
from plone.app.event.base import expand_events
from plone.app.layout.viewlets.common import PathBarViewlet
from plone.app.workflow.remap import remap_workflow
from plone.dexterity.interfaces import IDexterityFTI
from plone.event.interfaces import IOccurrence
from plone.resource.interfaces import IResourceDirectory
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.MailHost.interfaces import IMailHost
from zope.component import getSiteManager
from zope.component import getUtility
from zope.component import queryUtility
from zope.component.interfaces import ComponentLookupError
from zope.publisher.browser import BrowserView
from zope.ramcache.interfaces.ram import IRAMCache

import base64
import json
import logging
import zope.copy


logger = logging.getLogger('cpskin.core.browser.pages')


class FrontPage(BrowserView):

    index = ViewPageTemplateFile('templates/frontpage.pt')


class HelpPage(BrowserView):

    index = ViewPageTemplateFile('templates/helppage.pt')


class OpenData(BrowserView):
    """Get data news, events rss feed and collective.directory csv files."""

    index = ViewPageTemplateFile('templates/opendata.pt')

    def get_links(self):
        portal = api.portal.get()
        links = []
        path = '/'.join(portal.getPhysicalPath())
        portal_catalog = api.portal.get_tool('portal_catalog')
        query_dict = {}
        query_dict['path'] = {'query': path, 'depth': 2}
        query_dict['portal_type'] = ['Folder']
        query_dict['object_provides'] = IFVSC.__identifier__
        query_dict['sort_on'] = 'getObjPositionInParent'
        brains = portal_catalog(query_dict)
        for brain in brains:
            folder = brain.getObject()
            if getattr(folder, 'default_page', None):
                default_page = folder.default_page
                default_obj = folder[default_page]
                rsslink = '{0}/atom.xml'.format(default_obj.absolute_url())
                links.append(rsslink)

        query_dict = {}
        query_dict['portal_type'] = ['collective.directory.directory']
        brains = portal_catalog(query_dict)
        for brain in brains:
            directory = brain.getObject()
            catalog = api.portal.get_tool('portal_catalog')
            query_dict = {}
            query_dict['portal_type'] = 'collective.directory.card'
            query_dict['path'] = {
                'query': '/'.join(directory.getPhysicalPath()), 'depth': 3}
            size = len(catalog(query_dict))
            if size > 3:
                csvlink = '{0}/collective_directory_export_view'.format(
                    directory.absolute_url()
                )
                links.append(csvlink)

        return links


class IDocumentGenerationView(DocumentGenerationView):
    """Override the 'get_generation_context' properly so
       'get_base_generation_context' is available for sub-packages
       that want to extend the template generation context."""

    def _get_generation_context(self, helper_view, pod_template):
        result = super(IDocumentGenerationView, self)._get_generation_context(
            helper_view, pod_template)
        # if pod_template.portal_type == 'ConfigurablePODTemplate':
        if IFacetedNavigable.providedBy(self.context):
            brains = getDashboardQueryResult(self.context)
            result['brains'] = brains
            if len(brains) > 0 and brains[0].portal_type == 'Event':
                expandevents = expand_events(brains, 2)
                events = []
                for event in expandevents:
                    if IOccurrence.providedBy(event):
                        start = event.start
                        end = event.end
                        parent = zope.copy.copy(event.aq_parent.aq_base)
                        parent.start = start
                        parent.end = end
                        req = event.REQUEST
                        parent.REQUEST = req
                        parent.occurrence = True
                        parent.base_event = event.aq_parent
                        events.append(parent)
                    else:
                        events.append(event)
                result['brains'] = events
            return result


class EventGenerationHelperView(DXDocumentGenerationHelperView):

    def __init__(self, context, request):
        super(EventGenerationHelperView, self).__init__(context, request)
        if getattr(self.real_context, 'occurrence', False):
            self.dates = dates_for_display(self.real_context)
            self.real_context = self.real_context.base_event
        else:
            self.dates = dates_for_display(self.real_context)

    def is_same_month(self, start, end):
        return start.get('month') == end.get('month')

    def get_formatted_date(self):
        date_formated = u''
        event = self.real_context
        date_spel_start = date_speller(event, self.dates.get('start_iso'))
        date_spel_end = date_speller(event, self.dates.get('end_iso'))
        # day and month
        if self.dates.get('same_day'):
            date_formated = u'{0} {1} {2}'.format(
                date_spel_start.get('day'),
                date_spel_start.get('month'),
                date_spel_start.get('year'))
        elif self.is_same_month(date_spel_start, date_spel_end):
            date_formated = u'{0} au {1} {2} {3}'.format(
                date_spel_start.get('day'),
                date_spel_end.get('day'),
                date_spel_start.get('month'),
                date_spel_start.get('year'))
        else:
            date_formated += u'{0} {1} {2} au {3} {4} {5}'.format(
                date_spel_start.get('day'),
                date_spel_start.get('month'),
                date_spel_start.get('year'),
                date_spel_end.get('day'),
                date_spel_end.get('month'),
                date_spel_start.get('year'))

        # hour
        if not self.dates.get('whole_day'):
            if self.dates.get('open_end'):
                date_formated += _(u' à ')
                date_formated += u'{0}:{1}'.format(
                    date_spel_start.get('hour'),
                    date_spel_start.get('minute2'))
            else:
                date_formated += _(u' de ')
                date_formated += u'{0}:{1}'.format(
                    date_spel_start.get('hour'),
                    date_spel_start.get('minute2'))
                date_formated += _(u' à ')
                date_formated += u'{0}:{1}'.format(
                    date_spel_end.get('hour'),
                    date_spel_end.get('minute2')
                )
        return date_formated

    def get_values_in_one_line(self, values, sep=u' - '):
        results = [safe_unicode(value) for value in values if value]
        return sep.join(results)

    def get_taxonomy_values_in_one_line(self, field_names, sep, second_sep=' '):
        text = []
        for field_name in field_names:
            value = ''
            if isinstance(field_name, dict):
                (dict_text, dict_field_name) = field_name.items()[0]
                value = self.get_taxonomy_value(dict_field_name, second_sep)
                if value:
                    if self.hack_namur(field_name, value):
                        text.append('{0} {1}'.format(dict_text, value))
            else:
                value = self.get_taxonomy_value(field_name, second_sep)
                if value:
                    if self.hack_namur(field_name, value):
                        text.append(value)
        return sep.join(text)

    def hack_namur(self, field_name, value):
        """ This is developped for Namur agenda export only """
        if isinstance(field_name, dict):
            field_name = field_name.values()[0]
        if field_name.endswith('publiccible') and value == 'Tout public':
            return False
        if field_name.endswith('gratuite') and value != 'Gratuit':
            return False
            # if field_name.endswith('danslecadrede'):
            #     import ipdb; ipdb.set_trace()
        return True

    def get_taxonomy_value(self, field_name, sep=' ', end=''):
        lang = self.real_context.language
        taxonomy_ids = self.get_value(field_name)
        if not taxonomy_ids:
            return None
        if isinstance(taxonomy_ids, basestring):
            taxonomy_ids = [taxonomy_ids]
        domain = 'collective.taxonomy.{0}'.format(
            field_name.replace('taxonomy_', '').replace('_', ''))

        sm = getSiteManager()
        utility = sm.queryUtility(ITaxonomy, name=domain)
        taxonomy_list = [taxonomy_id for taxonomy_id in taxonomy_ids]
        text = []
        if len(taxonomy_list) > 0:
            for taxonomy_id in taxonomy_list:
                text.append(
                    safe_utf8(
                        utility.translate(
                            taxonomy_id,
                            context=self.real_context,
                            target_language=lang
                        )
                    )
                )
            return '{0}{1}'.format(sep.join(text), end)
        else:
            return None

    def get_relation_field(self, field_name):
        try:
            related_obj = self.get_value(field_name)
        except:
            return None
        if not related_obj:
            return None
        if isinstance(related_obj, str):
            return self.real_context
        if isinstance(related_obj, list):
            return [obj.to_object for obj in related_obj]
        return getattr(related_obj, 'to_object', None)

    def get_relation_value(self, field_name, value_name, sep=' ', end=''):
        if not getattr(self.real_context, field_name, None):
            return False
        if isinstance(value_name, list):
            result = []
            for value in value_name:
                relation_field = self.get_relation_field(field_name)
                result.append(getattr(relation_field, value, ''))
            return u'{0}{1}'.format(sep.join(result), end)
        relation_field = self.get_relation_field(field_name)
        if isinstance(relation_field, list):
            result = []
            for rel in relation_field:
                result.append(getattr(rel, value_name, ''))
            return u'{0}{1}'.format(sep.join(result), end)
        return u'{0}{1}'.format(getattr(relation_field, value_name, ''), end)

    def get_partners(self, prefix='', sep=' '):
        if not getattr(self.real_context, 'partners', None):
            return False
        partners = self.get_relation_value('partners', 'title', sep)
        text = partners
        if prefix:
            text = u'{0} {1}'.format(safe_unicode(prefix), partners)
        return text

    def display_phones(self, related_name='contact', field_name='phone'):
        obj = self.get_relation_field(related_name)
        phone = getattr(obj, field_name, None)
        result = ''
        if phone:
            if isinstance(phone, list):
                result = u' '.join([format_phone(p)['formated'] for p in phone])
            else:
                result = format_phone(phone)['formated']
        return result

    def get_info(self):
        if not getattr(self.real_context, 'contact', None):
            return False
        obj = self.get_relation_field('contact')
        info = []
        title = getattr(obj, 'title', None)
        if title:
            info.append(title)
        phone = getattr(obj, 'phone', None)
        if not phone:
            obj = self.get_relation_field('organizer')
            phone = getattr(obj, 'phone', None)
        if phone:
            if isinstance(phone, list):
                phone = u' '.join([format_phone(p)['formated'] for p in phone])
                # hack for namur
                phone = phone.replace('+32 (0) ', '0')
                info.append(phone)
            else:
                phone = format_phone(phone)['formated']
                # hack for namur
                phone = phone.replace('+32 (0) ', '0')
                info.append(phone)
        website = getattr(obj, 'website', None)
        if website:
            info.append(website)
        if len(info) >= 1:
            infomsg = _(u'Info :')
            return u'{0} {1}'.format(infomsg, ' - '.join(info))
        else:
            return ''

    def get_address(self):
        address = []
        address.append(_(u'Où ?'))
        title = self.get_relation_value('location', 'title')
        if title:
            address.append(title)
        street = self.get_relation_value('location', 'street')
        number = self.get_relation_value('location', 'number')
        if street:
            street_with_number = street
            if number:
                street_with_number = u'{0}, {1}'.format(street, str(number))
            address.append(street_with_number)
        zip_code = self.get_relation_value('location', 'zip_code')
        city = self.get_relation_value('location', 'city')
        if city and zip_code:
            address.append(u'{0} {1}'.format(zip_code, city))
        if len(address) > 1:
            return '\n'.join(address)
        else:
            return ''

    def get_image_from_text(self, field_name=None, text=None):
        if field_name:
            text = self.get_value(field_name)
        images = []
        import bs4
        soup = bs4.BeautifulSoup(text, 'html.parser')
        for img in soup.find_all('img'):
            images.append(img.get('src'))
        return images[0] if len(images) > 0 else images


class TupleErrorPage(BrowserView):
    def __call__(self):
        event_col = self.context
        queries = []
        for query in event_col.query:
            if not isinstance(query.get('v'), tuple):
                queries.append(query)
        event_col.query = queries
        return 'End of tuple error'


class TransmoExport(BrowserView):
    def __call__(self):
        objects = {}
        # get all file in custom folder
        portal_skins = api.portal.get_tool('portal_skins')
        objects['custom'] = []
        for obj_id, item in portal_skins.custom.items():
            if item.meta_type in ['Image', 'File']:
                try:
                    data = base64.b64encode(item.data)
                    objects['custom'].append({
                        'obj_id': obj_id,
                        'meta_type': item.meta_type,
                        'data': data
                    })
                except:
                    logger.info('Not able to export {0}'.format(
                        '/'.join(item.getPhysicalPath()))
                    )
            else:
                try:
                    objects['custom'].append({
                        'obj_id': obj_id,
                        'meta_type': item.meta_type,
                        'raw': item.raw
                    })
                except:
                    logger.info('Not able to export {0}'.format(
                        '/'.join(item.getPhysicalPath()))
                    )
        objects['default_skin'] = portal_skins.default_skin
        # get list of installed profile
        portal_quickinstaller = api.portal.get_tool('portal_quickinstaller')
        product_ids = [product['id'] for product in portal_quickinstaller.listInstalledProducts()]
        objects['products'] = product_ids

        portal_membership = api.portal.get_tool('portal_membership')
        list_members = portal_membership.listMembers()
        # groups
        groups = []
        for site_group in api.group.get_groups():
            group = {}
            group['id'] = site_group.getId()
            group['title'] = site_group.title
            group['description'] = site_group.description
            group['roles'] = site_group.getRoles()
            group['groups'] = site_group.getGroups()
            users = [user.id for user in api.user.get_users(groupname=group['id']) if user in list_members]  # noqa
            group['users'] = users
            groups.append(group)
        objects['groups'] = groups
        # users
        pas = api.portal.get().acl_users
        passwords = dict(pas.source_users._user_passwords)
        users = []
        for member in list_members:
            user = {}
            user['id'] = member.getId()
            user['name'] = member.getProperty('fullname', member.getUserName())
            user['email'] = member.getProperty('email', None)
            user['fullname'] = member.getProperty('fullname', None)
            login_time = member.getProperty('login_time', None)
            if login_time == DateTime('2000/01/01'):
                user['login_date'] = ''
            else:
                user['login_date'] = login_time.strftime('%d/%m/%Y')
            last_login_time = member.getProperty(
                'last_login_time',
                None
            )
            if last_login_time == DateTime('2000/01/01'):
                user['last_login_date'] = ''
            else:
                user['last_login_date'] = last_login_time.strftime('%d/%m/%Y')
            if user['login_date'] == user['last_login_date']:
                user['last_login_date'] = ''
            user['roles'] = member.getRoles()
            user['domains'] = member.getDomains()
            user['password'] = passwords.get(user['id'])
            users.append(user)
        objects['users'] = users

        # mail control panel
        mailhost = {}
        try:
            mail_host = getUtility(IMailHost)
        except ComponentLookupError:
            mail_host = getattr(api.portal.get(), 'MailHost')
        mailhost['smtp_host'] = mail_host.smtp_host
        mailhost['smtp_port'] = mail_host.smtp_port
        mailhost['smtp_userid'] = getattr(mail_host, 'smtp_userid', None)
        mailhost['smtp_uid'] = getattr(mail_host, 'smtp_uid', None)
        mailhost['smtp_pwd'] = mail_host.smtp_pwd
        mailhost['email_from_address'] = api.portal.get().email_from_address
        mailhost['email_from_name'] = api.portal.get().email_from_name
        objects['mailhost'] = mailhost

        # geo
        lat_key = 'collective.geo.settings.interfaces.IGeoSettings.latitude'
        lng_key = 'collective.geo.settings.interfaces.IGeoSettings.longitude'
        zoom_key = 'collective.geo.settings.interfaces.IGeoSettings.zoom'
        geo = {}
        geo['latitude'] = str(api.portal.get_registry_record(lat_key))
        geo['longitude'] = str(api.portal.get_registry_record(lng_key))
        geo['zoom'] = str(api.portal.get_registry_record(zoom_key))
        objects['geo'] = geo

        # portal_languages
        portal_languages = api.portal.get_tool('portal_languages')
        objects['languages'] = portal_languages.supported_langs

        # discussion
        discussion = {}
        anonymous_comments = 'plone.app.discussion.interfaces.IDiscussionSettings.anonymous_comments'  # bool
        anonymous_email_enabled = 'plone.app.discussion.interfaces.IDiscussionSettings.anonymous_email_enabled'   # bool
        captcha = 'plone.app.discussion.interfaces.IDiscussionSettings.captcha'  # choice
        edit_comment_enabled = 'plone.app.discussion.interfaces.IDiscussionSettings.edit_comment_enabled'  # bool
        globally_enabled = 'plone.app.discussion.interfaces.IDiscussionSettings.globally_enabled'  # bool
        moderation_enabled = 'plone.app.discussion.interfaces.IDiscussionSettings.moderation_enabled'  # bool
        moderator_email = 'plone.app.discussion.interfaces.IDiscussionSettings.moderator_email'  # textline
        moderator_notification_enabled = 'plone.app.discussion.interfaces.IDiscussionSettings.moderator_notification_enabled'  # bool
        show_commenter_image = 'plone.app.discussion.interfaces.IDiscussionSettings.show_commenter_image'  # bool
        user_notification_enabled = 'plone.app.discussion.interfaces.IDiscussionSettings.user_notification_enabled'  # bool
        discussion['anonymous_comments'] = str(api.portal.get_registry_record(anonymous_comments))
        discussion['anonymous_email_enabled'] = str(api.portal.get_registry_record(anonymous_email_enabled))
        discussion['captcha'] = api.portal.get_registry_record(captcha)
        discussion['edit_comment_enabled'] = str(api.portal.get_registry_record(edit_comment_enabled))
        discussion['globally_enabled'] = str(api.portal.get_registry_record(globally_enabled))
        discussion['moderation_enabled'] = str(api.portal.get_registry_record(moderation_enabled))
        discussion['moderator_email'] = str(api.portal.get_registry_record(moderator_email))
        discussion['moderator_notification_enabled'] = str(api.portal.get_registry_record(moderator_notification_enabled))
        discussion['show_commenter_image'] = str(api.portal.get_registry_record(show_commenter_image))
        discussion['user_notification_enabled'] = str(api.portal.get_registry_record(user_notification_enabled))
        objects['discussion'] = discussion

        portal_workflow = api.portal.get_tool('portal_workflow')
        objects['workflow'] = portal_workflow.getDefaultChain()[0]

        portal_resources = getUtility(IResourceDirectory, name='persistent')
        cpskin_resources_folder = portal_resources['cpskin']
        if cpskin_resources_folder:
            resources = {}
            for res in cpskin_resources_folder.listDirectory():
                resources[res] = cpskin_resources_folder[res].data
            objects['resources'] = resources

        # abonnes
        brains = api.content.find(portal_type='NewsletterTheme')
        newsletters = {}
        for brain in brains:
            obj = brain.getObject()
            path = brain.getPath()
            subscriber_brains = obj.getSubscribers()
            subscribers = []
            for subscriber_brain in subscriber_brains:
                subscriber_obj = subscriber_brain.getObject()
                subs = {}
                subs['active'] = subscriber_obj.active
                subs['email'] = subscriber_obj.email
                subs['fullname'] = subscriber_obj.fullname
                subs['format'] = subscriber_obj.format
                subscribers.append(subs)
            newsletters[path] = subscribers
        objects['newsletters'] = newsletters

        # leadimage
        portal_properties = api.portal.get_tool('portal_properties')
        cli_properties = portal_properties.get('cli_properties', None)
        if cli_properties:
            leadimage_settings = {}
            leadimage_settings['allowed_types'] = cli_properties.allowed_types
            objects['leadimage'] = leadimage_settings

        # behaviors
        portal_types = api.portal.get_tool('portal_types')
        behaviors = {}
        for portal_type in portal_types:
            fti = portal_types[portal_type]
            if IDexterityFTI.providedBy(fti):
                behaviors[portal_type] = fti.behaviors
        objects['behaviors'] = behaviors

        portal_catalog = api.portal.get_tool('portal_catalog')
        total_objects = len(portal_catalog({}))
        objects['total_objects'] = str(total_objects)
        response = self.request.response
        response.setHeader('Content-type', 'application/json')
        return json.dumps(objects)


class TeleService(BrowserView):
    index = ViewPageTemplateFile('templates/teleservice.pt')


class EmptyPathBarViewlet(PathBarViewlet):
    index = ViewPageTemplateFile('templates/empty.pt')


class CpskinHealthy(BrowserView):
    index = ViewPageTemplateFile('templates/cpskinhealthy.pt')

    def __init__(self, context, request):
        self.portal = api.portal.get()
        self.context = context
        self.request = request
        form = request.form
        if 'method' in form.keys():
            method = form['method']
            if method in dir(self):
                try:
                    self.__getattribute__(method)()
                except:
                    msg = 'You can not start method: {0}'.format(method)
            else:
                msg = 'Method "{0}" do not exists.'.format(method)

    def title(self):
        return _(u'Cpskin Healthy')

    def _redirect(self, msg=''):
        if self.request:
            if msg:
                api.portal.show_message(message=msg,
                                        request=self.request,
                                        type='info')
            url = '{0}/cpskinhealthy'.format(self.context.absolute_url())
            self.request.response.redirect(url)
        return msg

    def contacts(self):
        """Check if collective contact is installed and use cpskin workflow"""
        results = {}
        results['errors'] = []
        qi = api.portal.get_tool('portal_quickinstaller')
        product_ids = [product['id'] for product in qi.listInstalledProducts()]
        if 'collective.contact.core' in product_ids:
            results['is_installed'] = True
        else:
            results['is_installed'] = False
            return results
        workflow_id = self.get_workflow_id('organization')
        if workflow_id == 'collective_contact_core_workflow':
            results['is_cpskin_workflow'] = False
        else:
            results['is_cpskin_workflow'] = True
        return results

    def get_workflow_id(self, type_id):
        portal_workflow = api.portal.get_tool('portal_workflow')
        workflow = portal_workflow.getWorkflowsFor(type_id)
        if len(workflow) > 1:
            self._redirect('To much workflow for contacts.')
        workflow_id = workflow[0].id
        return workflow_id

    def install_contact_core(self):
        portal_setup = api.portal.get_tool('portal_setup')
        portal_setup.runAllImportStepsFromProfile(
            'collective.contact.core:default'
        )
        self._redirect('collective.contact.core installed.')

    def set_contact_worflow(self):
        portal = api.portal.get()
        workflow_id = self.get_workflow_id('organization')
        if workflow_id == 'cpskin_collective_contact_workflow':
            self._redirect('cpskin_collective_contact_workflow already set.')
        chain = ('cpskin_collective_contact_workflow',)
        types = ('held_position',
                 'organization',
                 'person',
                 'position')
        state_map = {'active': 'active',
                     'deactivated': 'deactivated'}
        remap_workflow(portal, type_ids=types, chain=chain,
                       state_map=state_map)
        util = queryUtility(IRAMCache)
        if util is not None:
            util.invalidateAll()
        self._redirect('cpskin_collective_contact_workflow set.')

    def get_site_language(self):
        return api.portal.get().language

    def is_site_language_fr(self):
        return self.get_site_language() == 'fr'

    def set_site_language(self):
        api.portal.get().language = 'fr'



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:copyright: (c) 2015 by Affinitic SPRL
:license: GPL, see LICENCE.txt for more details.
"""


from eea.facetednavigation.subtypes.interfaces import IPossibleFacetedNavigable


class ICPSkinPossibleFacetedNavigable(IPossibleFacetedNavigable):
    """
    Marker interface for all objects that should have the ability to be
    faceted navigable
    """



# -*- coding: utf-8 -*-

from Acquisition import aq_base
from copy import deepcopy
from cpskin.core.transmo.wrapper import Wrapper
from plone import api
from Products.Five.browser import BrowserView

import base64
import json
import pprint
import sys
import traceback


def _clean_dict(dct, error):
    new_dict = dct.copy()
    message = str(error)
    for key, value in dct.items():
        if message.startswith(repr(value)):
            del new_dict[key]
            return key, new_dict
    raise ValueError('Could not clean up object')


class GetItem(BrowserView):

    def __call__(self):
        """
        """

        try:
            context_dict = Wrapper(self.context)

        except Exception, e:
            tb = pprint.pformat(traceback.format_tb(sys.exc_info()[2]))
            return 'ERROR: exception wrapping object: {0}\n{1}'.format(
                str(e), tb)

        passed = False
        while not passed:
            try:
                JSON = json.dumps(context_dict)
                passed = True
            except Exception, error:
                if 'serializable' in str(error):
                    key, context_dict = _clean_dict(context_dict, error)
                    pprint.pprint('Not serializable member {0} of {1} ignored'.format(key, repr(self)))  # noqa
                    passed = False
                else:
                    return ('ERROR: Unknown error serializing object: {0}'.format(str(error)))  # noqa
        self.request.response.setHeader('Content-Type', 'application/json')
        return JSON


class GetChildren(BrowserView):

    def __call__(self):
        """
        """

        children = []
        if getattr(aq_base(self.context), 'objectIds', False):
            children = self.context.objectIds()
            # Btree based folders return an OOBTreeItems
            # object which is not serializable
            # Thus we need to convert it to a list
            if not isinstance(children, list):
                children = [item for item in children]

        self.request.response.setHeader('Content-Type', 'application/json')
        return json.dumps(children)


class GetCatalogResults(BrowserView):

    def __call__(self):
        """Returns a list of paths of all items found by the catalog.
           Query parameters can be passed in the request.
        """
        if not hasattr(self.context.aq_base, 'unrestrictedSearchResults'):
            return
        query = self.request.form.get('catalog_query', None)
        if query:
            query = eval(base64.b64decode(query),
                         {'__builtins__': None}, {})
        item_paths = [item.getPath() for item
                      in self.context.unrestrictedSearchResults(**query)]
        # sometimes some content are unindexed
        plone_path = '/'.join(api.portal.get().getPhysicalPath())
        item_paths_with_parents = add_parent(item_paths, plone_path)
        self.request.response.setHeader('Content-Type', 'application/json')
        return json.dumps(item_paths_with_parents)


def add_parent(item_paths, plone_path):
    old_paths = deepcopy(item_paths)
    paths = []
    for item_path in item_paths:
        parent_path = '/'.join(item_path.split('/')[:-1])
        if parent_path not in old_paths and parent_path not in paths \
                and parent_path is not plone_path:
            paths.append(parent_path)
        paths.append(item_path)
    return paths



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from cpskin.locales import CPSkinMessageFactory as _
from interfaces import ISendToManagerForm
from plone.formwidget.recaptcha.widget import ReCaptchaFieldWidget
from plone.z3cform import layout
from Products.CMFPlone.utils import pretty_title_or_id
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.MailHost.interfaces import IMailHost
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.interfaces import WidgetActionExecutionError
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter
from zope.component import getUtility

from zope.interface import Invalid


import logging

logger = logging.getLogger("Plone")


class SendToManagerForm(form.Form):
    label = _(
        u"report_error_to_site_manager", default=u"Report an error to the site manager"
    )

    description = _(
        u"provide_maximum_informations",
        default=u"Merci de fournir le maximum d'information ainsi que tout renseignement qui permette de la valider (source).",
    )

    fields = field.Fields(ISendToManagerForm)
    fields["captcha"].widgetFactory = ReCaptchaFieldWidget
    ignoreContext = True

    mail_template = ViewPageTemplateFile("templates/sendtomanager_mail_template.pt")

    @button.buttonAndHandler(_(u"label_send", default="Send"), name="send")
    def handle_send(self, action):
        data, errors = self.extractData()
        if errors:
            IStatusMessage(self.request).addStatusMessage(
                self.formErrorsMessage, type=u"error"
            )
            return

        portal_state = getMultiAdapter(
            (self.context, self.request), name=u"plone_portal_state"
        )
        site = portal_state.portal()

        send_from_address = data.get("send_from_address")
        send_to_address = site.getProperty("email_from_address", None)
        subject = _(u"error_reported", default=u"Error reported")
        title = pretty_title_or_id(self, self.context)
        description = self.context.Description()
        comment = data.get("comment", None)

        captcha = getMultiAdapter(
            (aq_inner(self.context), self.request), name="recaptcha"
        )
        if not captcha.verify():
            raise WidgetActionExecutionError(
                "captcha",
                Invalid(_(u"Please check the captcha to prove you're a human")),
            )

        try:
            # Sends a link of a page to someone.
            host = getUtility(IMailHost)
            encoding = site.getProperty("email_charset")

            # Cook from template
            message = self.mail_template(
                self,
                send_to_address=send_to_address,
                send_from_address=send_from_address,
                comment=comment,
                subject=subject,
                title=title,
                description=description,
            )

            message = message.encode(encoding)

            host.send(
                message,
                mto=send_to_address,
                mfrom=send_from_address,
                subject=subject,
                charset="utf-8",
            )

        except ConflictError:
            raise
        except Exception as e:
            # TODO To many things could possibly go wrong. So we catch all.
            logger.info("Unable to send mail: " + str(e))
            IStatusMessage(self.request).addStatusMessage(
                _(u"Unable to send mail."), type=u"error"
            )
            return

        IStatusMessage(self.request).addStatusMessage(_(u"Mail sent."), type=u"info")

        self.request.response.redirect(self.context.absolute_url())
        return ""


send_to_form = layout.wrap_form(SendToManagerForm)



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from Products.CMFPlone.utils import isDefaultPage
from Products.Five import BrowserView
from collective.folderishtypes.interfaces import IFolderishType
from plone import api


class UtilsView(BrowserView):

    def is_gdpr(self):
        return api.portal.get_registry_record(
            'imio.gdpr.interfaces.IGDPRSettings.is_text_ready',
            default=False)

    def has_folderish_default(self):
        if not IFolderishType.providedBy(self.context):
            return False
        return isDefaultPage(self.context, self.request)

    def convert_default_url(self, url, to_parent=False):
        """Converts url from default view url to parent or vice versa
        """
        obj_url = self.context.absolute_url()
        parent_url = aq_parent(self.context).absolute_url()
        if to_parent and obj_url in url:
            return url.replace(obj_url, parent_url)
        if not to_parent and obj_url not in url:
            return url.replace(parent_url, obj_url)
        return url



# -*- coding: utf-8 -*-
from cpskin.core.utils import image_scale
from plone import api
from plone.app.contenttypes.browser.folder import FolderView


class CpskinFolderView(FolderView):
    def __init__(self, context, request):
        request.set("limit_display", 100)
        super(CpskinFolderView, self).__init__(context, request)


class CpskinNavigationView(FolderView):
    def menus(self):
        portal_catalog = api.portal.get_tool("portal_catalog")
        context_path = "/".join(self.context.getPhysicalPath())
        query = {}
        query["review_state"] = "published_and_shown"
        query["path"] = {"query": context_path, "depth": 1}
        query["sort_on"] = "getObjPositionInParent"
        return portal_catalog(query)

    def accesses(self):
        if self.level() < 2:
            return []
        portal_catalog = api.portal.get_tool("portal_catalog")
        context_path = "/".join(self.context.getPhysicalPath())
        query = {}
        query["path"] = {"query": context_path, "depth": 10}
        query["sort_on"] = "sortable_title"
        query["object_provides"] = "cpskin.menu.interfaces.IDirectAccess"
        return portal_catalog(query)

    def level(self):
        navroot_level = len(
            api.portal.get_navigation_root(self.context).getPhysicalPath()
        )
        context_level = len(self.context.getPhysicalPath())
        return context_level - navroot_level


class CpskinNavigationViewWithLeadImage(FolderView):
    def menus(self):
        portal_catalog = api.portal.get_tool("portal_catalog")
        context_path = "/".join(self.context.getPhysicalPath())
        query = {}
        query["review_state"] = "published_and_shown"
        query["path"] = {"query": context_path, "depth": 1}
        query["sort_on"] = "getObjPositionInParent"
        return portal_catalog(query)

    def image(self, brain):
        obj = brain.getObject()
        image = image_scale(obj, "leadimage-navigation", "thumb", generate_tag=False)
        if image:
            return image.url



# -*- coding: utf-8 -*-
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from plone.app.testing import applyProfile

import unittest


class TestProfiles(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_uninstall(self):
        applyProfile(self.portal, 'cpskin.core:uninstall')

    def test_reinstall(self):
        applyProfile(self.portal, 'cpskin.core:uninstall')
        applyProfile(self.portal, 'cpskin.core:default')



# -*- coding: utf-8 -*-
from collective.contact.core.browser.basefields import views
from collective.contact.core.browser.organization import Organization
from collective.contact.core.browser.person import Person
from collective.contact.core.browser.position import Position
from collective.contact.core.content.organization import IOrganization
from cpskin.core.interfaces import ICPSkinCoreLayer
from five import grok


grok.templatedir('templates')


class OrganizationView(Organization):

    def _update(self):
        super(OrganizationView, self)._update()
        self.groups = tuple()


class PositionView(Position):

    def _update(self):
        super(PositionView, self)._update()
        self.groups = tuple(
            [group for group in self.groups if group.__name__ != 'coordinates']
        )


class PersonView(Person):

    def _update(self):
        super(PersonView, self)._update()
        self.groups = tuple(
            [group for group in self.groups if group.__name__ != 'coordinates']
        )


class OrganizationBaseFieldsView(views.OrganizationBaseFields):
    grok.name('basefields')
    grok.context(IOrganization)
    grok.template('organization_basefields')
    grok.layer(ICPSkinCoreLayer)



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from cpskin.core.browser.interfaces import IMediaActivationView
from cpskin.core.interfaces import IAlbumCollection
from cpskin.core.interfaces import IMediaActivated
from cpskin.core.interfaces import IVideoCollection
from cpskin.core.utils import publish_content
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import noLongerProvides


class MediaActivationView(BrowserView):
    """Media activation helper view"""
    implements(IMediaActivationView)

    def _redirect(self, msg=''):
        if self.request:
            if msg:
                api.portal.show_message(message=msg,
                                        request=self.request,
                                        type='info')
            self.request.response.redirect(self.context.absolute_url())
        return msg

    def _get_real_context(self):
        context = self.context
        plone_view = getMultiAdapter((context, self.request), name='plone')
        if plone_view.isDefaultPageInFolder():
            context = aq_parent(context)
        context = aq_inner(context)
        return context

    @property
    def is_enabled(self):
        # LATER : add caching here
        context = self._get_real_context()
        if IMediaActivated.providedBy(context):
            return True
        return False

    @property
    def can_enable_media(self):
        return not self.is_enabled

    @property
    def can_disable_media(self):
        context = self._get_real_context()
        return(IMediaActivated.providedBy(context))

    def enable_media(self):
        """Enable the media"""
        context = self._get_real_context()
        alsoProvides(context, IMediaActivated)
        catalog = api.portal.get_tool('portal_catalog')
        catalog.reindexObject(context)
        self._redirect(_(u'Multimedia viewlet enabled on content'))
        create_collections(context)

    def disable_media(self):
        """Disable the media"""
        context = self._get_real_context()
        noLongerProvides(context, IMediaActivated)
        catalog = api.portal.get_tool('portal_catalog')
        catalog.reindexObject(context)
        self._redirect(_(u'Multimedia viewlet disabled for content'))


def create_collections(context):
    """Create videos and albums collections if not exists."""
    catalog = api.portal.get_tool('portal_catalog')
    queryDict = {}
    queryDict['object_provides'] = IVideoCollection.__identifier__
    queryDict['path'] = {
        'query': '/'.join(context.getPhysicalPath()),
        'depth': 1
    }

    # --- Videos ---
    if not catalog(queryDict):
        video_folder = getattr(aq_base(context), 'videos', None)
        if not video_folder:
            video_folder = api.content.create(
                container=context,
                type='Folder',
                id='videos',
                title=u'Vidéos'
            )

        alsoProvides(video_folder, IVideoCollection)
        video_folder.reindexObject()

        video_collection = getattr(aq_base(video_folder), 'index', None)
        if not video_collection:
            video_collection = api.content.create(
                container=video_folder,
                type='Collection',
                id='index'
            )

        query = [
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['media_link']
            }, {
                'i': 'review_state',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['published_and_hidden', 'published_and_shown']
            }
        ]
        video_collection.query = query
        video_collection.sort_on = u'effective'
        video_collection.sort_reversed = True
        video_collection.setLayout('collection_oembed_view')

        publish_content(video_folder)
        publish_content(video_collection)

        video_folder.setDefaultPage('index')

    # --- Albums ---
    queryDict['object_provides'] = IAlbumCollection.__identifier__
    if not catalog(queryDict):
        album_folder = getattr(aq_base(context), 'albums', None)
        if not album_folder:
            album_folder = api.content.create(container=context,
                                              type='Folder',
                                              title=u'Albums')
        alsoProvides(album_folder, IAlbumCollection)
        album_folder.reindexObject()

        album_collection = getattr(aq_base(album_folder), 'index', None)
        if not album_collection:
            album_collection = api.content.create(
                container=album_folder,
                type='Collection',
                id='index'
            )

        query = [
            {
                'i': 'hiddenTags',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['multimedia-a-la-une']
            }, {
                'i': 'review_state',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['published_and_hidden', 'published_and_shown']
            }
        ]
        # path = '/'.join(album_folder.getPhysicalPath())
        album_collection.query = query
        album_collection.sort_on = u'effective'
        album_collection.sort_reversed = True

        publish_content(album_folder)
        publish_content(album_collection)

        album_folder.setDefaultPage('index')



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""
from eea.facetednavigation.interfaces import ICriteria
from eea.facetednavigation.layout.layout import FacetedLayout
from zope.component import queryAdapter
from zope.globalrequest import getRequest


class FacetedMultipleLayout(FacetedLayout):

    def __init__(self, *args, **kwargs):
        super(FacetedMultipleLayout, self).__init__(*args, **kwargs)
        self.request = getattr(self.context, 'REQUEST', getRequest())

    @property
    def layout(self):
        cid, criterion = self.get_criterion()
        default = criterion and criterion.default or None
        layout = self.request.form.get('%s[]' % cid, default)  # noqa
        if not layout:
            return super(FacetedMultipleLayout, self).layout
        return layout

    def get_criterion(self):
        criteria = queryAdapter(self.context, ICriteria)
        criterion = [(cid, c) for cid, c in criteria.items()
                     if c.widget == 'layout']
        return len(criterion) > 0 and criterion[0] or (None, None)



# -*- coding: utf-8 -*-
from cpskin.core.interfaces import ICPSkinSettings
from cpskin.core.interfaces import IElectedContentForTopMenu
from cpskin.core.utils import is_plone_app_multilingual_installed
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from plone.app.controlpanel.form import ControlPanelForm
from plone.app.multilingual.interfaces import ITranslationManager
from plone.registry.interfaces import IRegistry
from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapts
from zope.component import getUtility
from zope.formlib import form
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import noLongerProvides
from zope.schema.interfaces import IVocabularyFactory


class CPSkinControlPanelAdapter(SchemaAdapterBase):

    adapts(IPloneSiteRoot)
    implements(ICPSkinSettings)

    def __init__(self, context):
        super(CPSkinControlPanelAdapter, self).__init__(context)
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(ICPSkinSettings, False)

    def getLoadPageMenu(self):
        return self.settings.load_page_menu

    def setLoadPageMenu(self, value):
        self.settings.load_page_menu = value

    load_page_menu = property(getLoadPageMenu, setLoadPageMenu)

    def getSubMenuPersistence(self):
        return self.settings.sub_menu_persistence

    def setSubMenuPersistence(self, value):
        self.settings.sub_menu_persistence = value

    sub_menu_persistence = property(
        getSubMenuPersistence, setSubMenuPersistence)

    def getAutoPlaySlider(self):
        return self.settings.auto_play_slider

    def setAutoPlaySlider(self, value):
        self.settings.auto_play_slider = value

    auto_play_slider = property(getAutoPlaySlider, setAutoPlaySlider)

    def getSliderTimer(self):
        return self.settings.slider_timer

    def setSliderTimer(self, value):
        self.settings.slider_timer = value

    slider_timer = property(getSliderTimer, setSliderTimer)

    def getCityName(self):
        return self.settings.city_name

    def setCityName(self, value):
        self.settings.city_name = value

    city_name = property(getCityName, setCityName)

    def getContentsInActionMenu(self):
        return self.settings.contents_in_action_menu

    def setContentsInActionMenu(self, value):
        self.settings.contents_in_action_menu = value
        portal = root = api.portal.get()
        if 'fr' in root.objectIds():
            fr = getattr(root, 'fr')
            root = api.portal.get_navigation_root(fr)
        catalog = api.portal.get_tool('portal_catalog')
        name = 'cpskin.core.vocabularies.action_menu_eligible'
        factory = getUtility(IVocabularyFactory, name)
        vocabulary = factory(portal)
        all_values = vocabulary.by_value.keys()
        for content_id in all_values:
            content = getattr(root, content_id, None)
            if not content:
                continue
            translations = {'fr': content}
            request = getattr(self.context, 'REQUEST', None)
            if is_plone_app_multilingual_installed(request):
                translations = ITranslationManager(content).get_translations()
            if content_id in value:
                for t in translations.values():
                    if IElectedContentForTopMenu.providedBy(t):
                        continue
                    alsoProvides(t, IElectedContentForTopMenu)
                    catalog.reindexObject(t)
            else:
                for t in translations.values():
                    if not IElectedContentForTopMenu.providedBy(t):
                        continue
                    noLongerProvides(t, IElectedContentForTopMenu)
                    catalog.reindexObject(t)

    contents_in_action_menu = property(
        getContentsInActionMenu,
        setContentsInActionMenu,
    )

    def getShowLeadimageInActionMenu(self):
        return self.settings.show_leadimage_in_action_menu

    def setShowLeadimageInActionMenu(self, value):
        self.settings.show_leadimage_in_action_menu = value

    show_leadimage_in_action_menu = property(
        getShowLeadimageInActionMenu,
        setShowLeadimageInActionMenu,
    )

    def getPersonContactCoreFallback(self):
        return self.settings.person_contact_core_fallback

    def setPersonContactCoreFallback(self, value):
        self.settings.person_contact_core_fallback = value

    person_contact_core_fallback = property(
        getPersonContactCoreFallback,
        setPersonContactCoreFallback,
    )

    def getShowSlogan(self):
        return self.settings.show_slogan

    def setShowSlogan(self, value):
        self.settings.show_slogan = value

    show_slogan = property(
        getShowSlogan,
        setShowSlogan,
    )

    def getshowPortletsInRightActionsPanel(self):
        return self.settings.show_portlets_in_right_actions_panel

    def setshowPortletsInRightActionsPanel(self, value):
        self.settings.show_portlets_in_right_actions_panel = value

    show_portlets_in_right_actions_panel = property(
        getshowPortletsInRightActionsPanel,
        setshowPortletsInRightActionsPanel,
    )

    def getMediaViewletVisibleAlbums(self):
        return self.settings.media_viewlet_visible_albums

    def setMediaViewletVisibleAlbums(self, value):
        self.settings.media_viewlet_visible_albums = value

    media_viewlet_visible_albums = property(
        getMediaViewletVisibleAlbums,
        setMediaViewletVisibleAlbums,
    )

    def getMediaViewletVisibleVideos(self):
        return self.settings.media_viewlet_visible_videos

    def setMediaViewletVisibleVideos(self, value):
        self.settings.media_viewlet_visible_videos = value

    media_viewlet_visible_videos = property(
        getMediaViewletVisibleVideos,
        setMediaViewletVisibleVideos,
    )

    def getShowDescriptionOnThemes(self):
        return self.settings.show_description_on_themes

    def setShowDescriptionOnThemes(self, value):
        self.settings.show_description_on_themes = value

    show_description_on_themes = property(
        getShowDescriptionOnThemes,
        setShowDescriptionOnThemes,
    )

    def getSearchPosition(self):
        return self.settings.search_position

    def setSearchPosition(self, value):
        self.settings.search_position = value

    search_position = property(getSearchPosition, setSearchPosition)

    def getCollapseMinisiteMenu(self):
        return self.settings.collapse_minisite_menu

    def setCollapseMinisiteMenu(self, value):
        self.settings.collapse_minisite_menu = value

    collapse_minisite_menu = property(
        getCollapseMinisiteMenu,
        setCollapseMinisiteMenu,
    )

    def getShowFooterSitemap(self):
        return self.settings.show_footer_sitemap

    def setShowFooterSitemap(self, value):
        self.settings.show_footer_sitemap = value

    show_footer_sitemap = property(
        getShowFooterSitemap,
        setShowFooterSitemap,
    )

    def getHeaderClass(self):
        return self.settings.header_class

    def setHeaderClass(self, value):
        self.settings.header_class = value

    header_class = property(getHeaderClass, setHeaderClass)

    def getNavigationClass(self):
        return self.settings.navigation_class

    def setNavigationClass(self, value):
        self.settings.navigation_class = value

    navigation_class = property(getNavigationClass, setNavigationClass)

    def getColumnsClass(self):
        return self.settings.columns_class

    def setColumnsClass(self, value):
        self.settings.columns_class = value

    columns_class = property(getColumnsClass, setColumnsClass)

    def getFooterClass(self):
        return self.settings.footer_class

    def setFooterClass(self, value):
        self.settings.footer_class = value

    footer_class = property(getFooterClass, setFooterClass)

    def getIndexedTaxonomies(self):
        return self.settings.indexed_taxonomies

    def setIndexedTaxonomies(self, value):
        self.settings.indexed_taxonomies = value

    indexed_taxonomies = property(getIndexedTaxonomies, setIndexedTaxonomies)

    @property
    def use_slick(self):
        return self.settings.use_slick

    @use_slick.setter
    def use_slick(self, value):
        self.settings.use_slick = value


class CPSkinControlPanel(ControlPanelForm):

    label = _('CPSkin settings')
    description = _('Lets you change the settings of CPSkin')
    form_fields = form.FormFields(ICPSkinSettings)



# -*- coding: utf-8 -*-
# from cpskin.core.utils import get_address_from_obj
from cpskin.citizen.utils import execute_under_unrestricted_user
from cpskin.core.utils import has_lat_lng
from cpskin.core.utils import set_coord
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest
import logging
import transaction

logger = logging.getLogger('cpskin.core')


def set_lat_lng(obj, event):
    """Set lat and lng when a organization is created"""
    if has_lat_lng(obj):
        return
    request = obj.REQUEST
    set_coord(obj, request)


def apply_crops_after_modify(obj, event):
    """
    Bug fixed here : cropped images scales on a content are lost after a
                     modification of this content (#14901).
    This is already fixed in Plone 5 but not in Plone 4 :
        https://github.com/collective/plone.app.imagecropping/issues/21
    To fix this, we need to re-generate all the crops of an object just after
    it's modification.

    We also need to handle fieldnames conflicting with :
      other field name + _ + scale name
      (ex : 'image_banner' field and 'image' field with 'banner' scale)
    """
    crops = IAnnotations(obj).get(PAI_STORAGE_KEY)
    if not crops:
        return
    croputils = IImageCroppingUtils(obj)
    request = getRequest()
    cropper = getMultiAdapter((obj, request), name='crop-image')
    for fieldname in croputils.image_field_names():
        for crop_key in crops:
            if crop_key.startswith(fieldname) and len(crop_key) > len(fieldname):
                if crop_key[len(fieldname)] != "_":
                    continue
                scalename = crop_key[len(fieldname) + 1:]
                try:
                    cropper._crop(fieldname, scalename, crops[crop_key])
                except KeyError:
                    # Handles special cases with field / scales collisions
                    continue


def checkMinisites(event):
    minisite_root = event.minisite
    footer_doc = getattr(minisite_root, 'footer-mini-site', None)
    if footer_doc:
        logger.info('Found footer-mini-site doc for {0}'.format(
            minisite_root.absolute_url(),
        ))
        return
    footer_doc = execute_under_unrestricted_user(
        api.portal.get(),
        api.content.create,
        'admin',
        type='Document',
        id='footer-mini-site',
        title='Footer',
        container=minisite_root
    )
    logger.info('Created footer-mini-site doc for {0}'.format(
        minisite_root.absolute_url(),
    ))
    transaction.commit()



# -*- coding: utf-8 -*-
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import format_phone
from cpskin.core.utils import set_plonecustom_last
from plone import api

import unittest


class TestTools(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_plonecustom_to_last(self):
        portal_css = api.portal.get_tool('portal_css')
        resources = list(portal_css.resources)
        self.assertFalse(resources[-1].getId() == 'ploneCustom.css')
        set_plonecustom_last()
        resources = list(portal_css.resources)
        self.assertTrue(resources[-1].getId() == 'ploneCustom.css')

    def test_format_phone(self):
        good_phone = '+32 (0) 65 32 96 77'
        formated_phone = format_phone('065/32.96.77').get('formated')
        self.assertEqual(formated_phone, good_phone)

        formated_phone = format_phone('65329677').get('formated')
        self.assertEqual(formated_phone, good_phone)

        formated_phone = format_phone('003265/32.96.77').get('formated')
        self.assertEqual(formated_phone, good_phone)

        formated_phone = format_phone('+3265329677').get('formated')
        self.assertEqual(formated_phone, good_phone)
